#!/usr/bin/python3
# -*- coding: utf-8 -*-

# desc.py file is part of slpkg.

# Copyright 2014-2022 Dimitris Zlatanidis <d.zlatanidis@gmail.com>
# All rights reserved.

# Slpkg is a user-friendly package manager for Slackware installations

# https://gitlab.com/dslackw/slpkg

# Slpkg is free software: you can redistribute it and/or modify
# it under the terms of the GNU General Public License as published by
# the Free Software Foundation, either version 3 of the License, or
# (at your option) any later version.
# This program is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. See the
# GNU General Public License for more details.
# You should have received a copy of the GNU General Public License
# along with this program. If not, see <http://www.gnu.org/licenses/>.


from slpkg.utils import Utils
from slpkg.messages import Msg
from slpkg.__metadata__ import MetaData as _meta_

from slpkg.sbo.greps import SBoGrep


class PkgDesc(Utils):
    """Print package description from the repository
    """
    def __init__(self, name, repo, paint):
        self.name = name
        self.repo = repo
        self.paint = paint
        self.meta = _meta_
        self.msg = Msg()
        self.lib = ""
        self.color = {
            "red": self.meta.color["RED"],
            "green": self.meta.color["GREEN"],
            "yellow": self.meta.color["YELLOW"],
            "cyan": self.meta.color["CYAN"],
            "grey": self.meta.color["GREY"],
            "": ""
        }[self.paint]
        if self.repo in self.meta.repositories and self.repo != "sbo":
            self.lib = f"{self.meta.lib_path}{self.repo}_repo/PACKAGES.TXT"

    def view(self):
        """Print package description by repository
        """
        description, count = "", 0
        if self.repo == "sbo":
            description = SBoGrep(self.name).description()
        else:
            PACKAGES_TXT = self.read_file(self.lib)
            for line in PACKAGES_TXT.splitlines():
                if line.startswith(self.name + ":"):
                    description += f"{line[len(self.name) + 2:]}\n"
                    count += 1
                    if count == 11:
                        break
        if description:
            print(f"{self.color}{description}{self.meta.color['ENDC']}")
        else:
            self.msg.pkg_not_found("", self.name, "No matching", "")


#!/usr/bin/python3
# -*- coding: utf-8 -*-

# build_num.py file is part of slpkg.

# Copyright 2014-2022 Dimitris Zlatanidis <d.zlatanidis@gmail.com>
# All rights reserved.

# Slpkg is a user-friendly package manager for Slackware installations

# https://gitlab.com/dslackw/slpkg

# Slpkg is free software: you can redistribute it and/or modify
# it under the terms of the GNU General Public License as published by
# the Free Software Foundation, either version 3 of the License, or
# (at your option) any later version.
# This program is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. See the
# GNU General Public License for more details.
# You should have received a copy of the GNU General Public License
# along with this program. If not, see <http://www.gnu.org/licenses/>.


import re

from slpkg.utils import Utils
from slpkg.url_read import URL
from slpkg.__metadata__ import MetaData as _meta_


class BuildNumber(Utils):
    """Get build number from SlackBuild script
    """
    def __init__(self, sbo_url, pkg):
        self.sbo_url = sbo_url
        self.pkg = pkg
        self.meta = _meta_

    def get(self):
        num = "NO_BUILD"
        if self.sbo_url:
            SlackBuild = URL(f"{self.sbo_url}"
                             f"{self.pkg}.SlackBuild").get_request()
        else:
            SlackBuild = self.read_file(f"{self.meta.build_path}{self.pkg}/"
                                        f"{self.pkg}.SlackBuild")
        for line in SlackBuild.splitlines():
            line = line.lstrip()
            if line.startswith("BUILD="):
                num = re.findall(r"\d+", line)
                break
        return "".join(num)



#!/usr/bin/python3
# -*- coding: utf-8 -*-

# sizes.py file is part of slpkg.

# Copyright 2014-2022 Dimitris Zlatanidis <d.zlatanidis@gmail.com>
# All rights reserved.

# Slpkg is a user-friendly package manager for Slackware installations

# https://gitlab.com/dslackw/slpkg

# Slpkg is free software: you can redistribute it and/or modify
# it under the terms of the GNU General Public License as published by
# the Free Software Foundation, either version 3 of the License, or
# (at your option) any later version.
# This program is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. See the
# GNU General Public License for more details.
# You should have received a copy of the GNU General Public License
# along with this program. If not, see <http://www.gnu.org/licenses/>.


def units(comp_sum, uncomp_sum):
    """Calculate package size
    """
    compressed = round((sum(map(float, comp_sum)) / 1024), 2)
    uncompressed = round((sum(map(float, uncomp_sum)) / 1024), 2)
    comp_unit = uncomp_unit = "Mb"
    if compressed > 1024:
        compressed = round((compressed / 1024), 2)
        comp_unit = "Gb"
    if uncompressed > 1024:
        uncompressed = round((uncompressed / 1024), 2)
        uncomp_unit = "Gb"
    if compressed < 1:
        compressed = sum(map(int, comp_sum))
        comp_unit = "Kb"
    if uncompressed < 1:
        uncompressed = sum(map(int, uncomp_sum))
        uncomp_unit = "Kb"
    return [comp_unit, uncomp_unit], [compressed, uncompressed]



#!/usr/bin/python3
# -*- coding: utf-8 -*-

# check.py file is part of slpkg.

# Copyright 2014-2022 Dimitris Zlatanidis <d.zlatanidis@gmail.com>
# All rights reserved.

# Slpkg is a user-friendly package manager for Slackware installations

# https://gitlab.com/dslackw/slpkg

# Slpkg is free software: you can redistribute it and/or modify
# it under the terms of the GNU General Public License as published by
# the Free Software Foundation, either version 3 of the License, or
# (at your option) any later version.
# This program is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. See the
# GNU General Public License for more details.
# You should have received a copy of the GNU General Public License
# along with this program. If not, see <http://www.gnu.org/licenses/>.


import os
from pkg_resources import parse_version

from slpkg.messages import Msg
from slpkg.blacklist import BlackList
from slpkg.splitting import split_package
from slpkg.upgrade_checklist import choose_upg
from slpkg.__metadata__ import MetaData as _meta_

from slpkg.sbo.greps import SBoGrep


def sbo_upgrade(skip, flag):
    """Return packages for upgrade
    """
    msg = Msg()
    black = BlackList()
    msg.checking()
    upgrade_names = []
    data = SBoGrep(name="").names()
    blacklist = list(black.get_black())
    for pkg in sbo_list():
        name = split_package(pkg)[0]
        ver = split_package(pkg)[1]
        if (name in data and name not in skip and name not in blacklist):
            sbo_package = f"{name}-{SBoGrep(name).version()}"
            package = f"{name}-{ver}"
            if parse_version(sbo_package) > parse_version(package):
                upgrade_names.append(name)
    msg.done()
    if "--checklist" in flag:
        upgrade_names = choose_upg(upgrade_names)
    return upgrade_names


def sbo_list():
    """Return all SBo packages
    """
    for pkg in os.listdir(_meta_.pkg_path):
        if pkg.endswith("_SBo"):
            yield pkg



import unittest
from slpkg.slack.slack_version import slack_ver


class TestSlackVersion(unittest.TestCase):

    def test_slack_version(self):
        """Testing the current Slackware version
        """
        ver = '15.0'
        self.assertEqual(ver, slack_ver())


if __name__ == "__main__":
    unittest.main()



#!/usr/bin/python3
# -*- coding: utf-8 -*-

# dependency.py file is part of slpkg.

# Copyright 2014-2022 Dimitris Zlatanidis <d.zlatanidis@gmail.com>
# All rights reserved.

# Slpkg is a user-friendly package manager for Slackware installations

# https://gitlab.com/dslackw/slpkg

# Slpkg is free software: you can redistribute it and/or modify
# it under the terms of the GNU General Public License as published by
# the Free Software Foundation, either version 3 of the License, or
# (at your option) any later version.
# This program is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. See the
# GNU General Public License for more details.
# You should have received a copy of the GNU General Public License
# along with this program. If not, see <http://www.gnu.org/licenses/>.


import sys
from functools import lru_cache

from slpkg.__metadata__ import MetaData as _meta_

from slpkg.binary.greps import Requires


class Dependencies:
    """Resolving binary dependencies
    """
    def __init__(self, repo, black):
        self.repo = repo
        self.black = black
        self.dep_results = []
        self.meta = _meta_

    def binary(self, name, flag):
        """Build all dependencies of a package
        """
        if self.meta.rsl_deps in ["on", "ON"] and "--resolve-off" not in flag:
            sys.setrecursionlimit(10000)
            dependencies = []
            requires = Requires(name, self.repo).get_deps()
            if requires:
                for req in requires:
                    if req and req not in self.black:
                        dependencies.append(req)
                self.deep_check(tuple(dependencies), tuple(flag))
            return self.dep_results
        else:
            return []

    @lru_cache
    def deep_check(self, dependencies, flag):
        """Checking if dependencies are finnished
        """
        if dependencies:
            self.dep_results.append(dependencies)
            [self.binary(dep, flag) for dep in dependencies]



#!/usr/bin/python3
# -*- coding: utf-8 -*-

# config.py file is part of slpkg.

# Copyright 2014-2022 Dimitris Zlatanidis <d.zlatanidis@gmail.com>
# All rights reserved.

# Slpkg is a user-friendly package manager for Slackware installations

# https://gitlab.com/dslackw/slpkg

# Slpkg is free software: you can redistribute it and/or modify
# it under the terms of the GNU General Public License as published by
# the Free Software Foundation, either version 3 of the License, or
# (at your option) any later version.
# This program is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. See the
# GNU General Public License for more details.
# You should have received a copy of the GNU General Public License
# along with this program. If not, see <http://www.gnu.org/licenses/>.


import shutil
import filecmp
import subprocess

from slpkg.utils import Utils
from slpkg.__metadata__ import MetaData as _meta_


class Config(Utils):
    """Print or edit slpkg configuration file
    """
    def __init__(self):
        self.config_file = "/etc/slpkg/slpkg.conf"
        self.meta = _meta_
        self.green = _meta_.color["GREEN"]
        self.red = _meta_.color["RED"]
        self.cyan = _meta_.color["CYAN"]
        self.endc = _meta_.color["ENDC"]

    def view(self):
        """View slpkg config file
        """
        conf_args = [
            "RELEASE",
            "SLACKWARE_VERSION",
            "COMP_ARCH",
            "BUILD_PATH",
            "PACKAGES",
            "PATCHES",
            "CHECKMD5",
            "DEL_ALL",
            "DEL_BUILD",
            "SBO_BUILD_LOG",
            "MAKEFLAGS",
            "DEFAULT_ANSWER",
            "REMOVE_DEPS_ANSWER",
            "SKIP_UNST",
            "RSL_DEPS",
            "DEL_DEPS",
            "USE_COLORS",
            "DOWNDER",
            "DOWNDER_OPTIONS",
            "SLACKPKG_LOG",
            "ONLY_INSTALLED",
            "EDITOR",
            "NOT_DOWNGRADE",
            "HTTP_PROXY",
        ]
        read_conf = self.read_file(self.config_file)
        for line in read_conf.splitlines():
            if not line.startswith("#") and line.split("=")[0] in conf_args:
                print(line)
            else:
                print(f"{self.cyan}{line}{self.endc}", end="\n")

    def edit(self):
        """Edit configuration file
        """
        subprocess.call(f"{self.meta.editor} {self.config_file}", shell=True)

    def reset(self):
        """Reset slpkg.conf file with default values
        """
        shutil.copy2(f"{self.config_file}.orig", self.config_file)
        if filecmp.cmp(f"{self.config_file}.orig", self.config_file):
            print(f"{self.green}The reset was done{self.endc}")
        else:
            print(f"{self.red}Reset failed{self.endc}")


#!/usr/bin/python3
# -*- coding: utf-8 -*-

# utils.py file is part of slpkg.

# Copyright 2014-2022 Dimitris Zlatanidis <d.zlatanidis@gmail.com>
# All rights reserved.

# Slpkg is a user-friendly package manager for Slackware installations

# https://gitlab.com/dslackw/slpkg

# Slpkg is free software: you can redistribute it and/or modify
# it under the terms of the GNU General Public License as published by
# the Free Software Foundation, either version 3 of the License, or
# (at your option) any later version.
# This program is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. See the
# GNU General Public License for more details.
# You should have received a copy of the GNU General Public License
# along with this program. If not, see <http://www.gnu.org/licenses/>.


import os
from collections import OrderedDict

from slpkg.splitting import split_package


class Utils:
    """Class with usefull utilities
    """
    def case_sensitive(self, lst):
        """Create dictionary from list with key in lower case
        and value with default
        """
        dictionary = {}
        for pkg in lst:
            dictionary[pkg.lower()] = pkg
        return dictionary

    def dimensional_list(self, lists):
        """Create one dimensional list
        """
        one_list = []
        for lst in lists:
            one_list += lst
        return one_list

    def remove_dbs(self, double):
        """Remove double item from list
        """
        return list(OrderedDict.fromkeys(double))

    def read_file(self, registry):
        """Returns reading file
        """
        code = self.check_encoding('', registry)
        if not code:
            code = "utf-8"
        with open(registry, "r", encoding=code) as file_txt:
            read_file = file_txt.read()
            return read_file

    def package_name(self, PACKAGES_TXT):
        """Returns list with all the names of packages repository
        """
        for line in PACKAGES_TXT.splitlines():
            if line.startswith("PACKAGE NAME:"):
                yield split_package(line[14:].strip())[0]

    def check_downloaded(self, path, downloaded):
        """Check if files downloaded and return downloaded
        packages
        """
        for pkg in downloaded:
            if os.path.isfile(f"{path}{pkg}"):
                yield pkg

    def read_config(self, config):
        """Read config file and returns first uncomment line
        and stop. Used for Slackware mirrors
        """
        for line in config.splitlines():
            line = line.lstrip()
            if line and not line.startswith("#"):
                return line

    def fix_file_name(self, file_name):
        """Get file name from url and fix passing char '+'
        """
        if "%2b" in file_name:
            return file_name.replace("%2b", "+", 5)
        elif "%2B" in file_name:
            return file_name.replace("%2B", "+", 5)
        else:
            return file_name

    def check_encoding(self, path, f):
        """Checking the file encoding default is utf-8
        """
        try:
            with open(f"{path}{f}", "r") as ftest:
                ftest.read()
        except UnicodeDecodeError:
            return "ISO-8859-1"

    def debug(self, test):
        """Function used for print some stuff for debugging
        """
        print(test)



#!/usr/bin/python3
# -*- coding: utf-8 -*-

# slack_version.py file is part of slpkg.

# Copyright 2014-2022 Dimitris Zlatanidis <d.zlatanidis@gmail.com>
# All rights reserved.

# Slpkg is a user-friendly package manager for Slackware installations

# https://gitlab.com/dslackw/slpkg

# Slpkg is free software: you can redistribute it and/or modify
# it under the terms of the GNU General Public License as published by
# the Free Software Foundation, either version 3 of the License, or
# (at your option) any later version.
# This program is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. See the
# GNU General Public License for more details.
# You should have received a copy of the GNU General Public License
# along with this program. If not, see <http://www.gnu.org/licenses/>.


import re

from slpkg.utils import Utils
from slpkg.__metadata__ import MetaData as _meta_


def slack_ver():
    """
    Open file and read Slackware version
    """
    utils = Utils()
    if _meta_.slackware_version in ["off", "OFF"]:
        sv = utils.read_file("/etc/slackware-version")
        version = re.findall(r"\d+", sv)
        if len(sv) > 2:
            return (".".join(version[:2]))
        else:
            return (".".join(version))
    return _meta_.slackware_version



import unittest
from slpkg.sizes import units


class TetsUnits(unittest.TestCase):

    def test_units(self):
        """Testing the units metrics
        """
        self.assertCountEqual((["Kb", "Kb"], [100, 100]),
                              units(["100"], ["100"]))
        self.assertCountEqual((["Mb", "Mb"], [1000, 1000]),
                              units(["1024000"], ["1024000"]))
        self.assertCountEqual((["Gb", "Gb"], [976.56, 976.56]),
                              units(["1024000000"], ["1024000000"]))


if __name__ == "__main__":
    unittest.main()



#!/usr/bin/python3
# -*- coding: utf-8 -*-

# new_config.py file is part of slpkg.

# Copyright 2014-2022 Dimitris Zlatanidis <d.zlatanidis@gmail.com>
# All rights reserved.

# Slpkg is a user-friendly package manager for Slackware installations

# https://gitlab.com/dslackw/slpkg

# Slpkg is free software: you can redistribute it and/or modify
# it under the terms of the GNU General Public License as published by
# the Free Software Foundation, either version 3 of the License, or
# (at your option) any later version.
# This program is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. See the
# GNU General Public License for more details.
# You should have received a copy of the GNU General Public License
# along with this program. If not, see <http://www.gnu.org/licenses/>.


import os
import shutil
import itertools

from slpkg.messages import Msg
from slpkg.utils import Utils
from slpkg.__metadata__ import MetaData as _meta_


class NewConfig(Utils):
    """Manage .new configuration files
    """
    def __init__(self):
        self.meta = _meta_
        self.msg = Msg()
        self.red = self.meta.color["RED"]
        self.green = self.meta.color["GREEN"]
        self.endc = self.meta.color["ENDC"]
        self.br = ""
        if self.meta.use_colors in ["off", "OFF"]:
            self.br = ")"
        self.etc = "/etc/"
        self.news = []

    def run(self):
        """print .new configuration files
        """
        self.find_new()
        for n in self.news:
            print(n)
        print()
        self.msg.template(78)
        print(f"| Installed {len(self.news)} new configuration files:")
        self.msg.template(78)
        self.choices()

    def find_new(self):
        """Find all '.new' files from /etc/ folder
        and subfolders
        """
        print("Search for .new configuration files:\n")
        for path, dirs, files in os.walk(self.etc):
            del dirs    # delete unsed
            for f in files:
                if f.endswith(".new"):
                    self.news.append(os.path.join(path, f))
        if not self.news:
            print("  No new configuration files\n")
            raise SystemExit()

    def choices(self):
        """Menu options for new configuration files
        """
        print(f"| {self.red}K{self.endc}{self.br}eep the old and .new files, no changes")
        print(f"| {self.red}O{self.endc}{self.br}verwrite all old configuration files with new ones")
        print("|  The old files will be saved with suffix .old")
        print(f"| {self.red}R{self.endc}{self.br}emove all .new files")
        print(f"| {self.red}P{self.endc}{self.br}rompt K, O, R, D, M option for each single file")
        print(f"| {self.red}Q{self.endc}{self.br}uit from menu")
        self.msg.template(78)
        try:
            choose = input("\nWhat would you like to do [K/O/R/P/Q]? ")
        except EOFError:
            raise SystemExit("\n")
        print()
        if choose in ("K", "k"):
            self.keep()
        elif choose in ("O", "o"):
            self.overwrite_all()
        elif choose in ("R", "r"):
            self.remove_all()
        elif choose in ("P", "p"):
            self.prompt()

    def overwrite_all(self):
        """Overwrite all .new files and keep
        old with suffix .old
        """
        for n in self.news:
            self._overwrite(n)

    def remove_all(self):
        """Remove all .new files
        """
        for n in self.news:
            self._remove(n)
        print()

    def prompt(self):
        """Select file
        """
        self.msg.template(78)
        print("| Choose what to do file by file:")
        print("| {0}K{1}{2}eep, {3}O{4}{5}verwrite, {6}R{7}{8}emove, "
              "{9}D{10}{11}iff, {12}M{13}{14}erge, {15}Q{16}{17}uit".format(
                  self.red, self.endc, self.br, self.red, self.endc, self.br,
                  self.red, self.endc, self.br, self.red, self.endc, self.br,
                  self.red, self.endc, self.br, self.red, self.endc, self.br))
        self.msg.template(78)
        print()
        self.i = 0
        try:
            while self.i < len(self.news):
                self.question(self.news[self.i])
                self.i += 1
        except EOFError:
            raise SystemExit("\n")

    def question(self, n):
        """Choose what do to file by file
        """
        print()
        prompt_ask = input(f"{n} [K/O/R/D/M/Q]? ")
        print()
        if prompt_ask in ("K", "k"):
            self.keep()
        elif prompt_ask in ("O", "o"):
            self._overwrite(n)
        elif prompt_ask in ("R", "r"):
            self._remove(n)
        elif prompt_ask in ("D", "d"):
            self.diff(n)
            self.i -= 1
        elif prompt_ask in ("M", "m"):
            self.merge(n)
        elif prompt_ask in ("Q", "q", "quit"):
            self.quit()

    def _remove(self, n):
        """Remove one single file
        """
        if os.path.isfile(n):
            os.remove(n)
        if not os.path.isfile(n):
            print(f"File '{n}' removed")

    def _overwrite(self, n):
        """Overwrite old file with new and keep file with suffix .old
        """
        if os.path.isfile(n[:-4]):
            shutil.copy2(n[:-4], n[:-4] + ".old")
            print("Old file {0} saved as {1}.old".format(
                n[:-4].split("/")[-1], n[:-4].split("/")[-1]))
        if os.path.isfile(n):
            shutil.move(n, n[:-4])
            print("New file {0} overwrite as {1}".format(
                n.split("/")[-1], n[:-4].split("/")[-1]))

    def keep(self):
        pass

    def diff(self, n):
        """Print the differences between the two files
        """
        if os.path.isfile(n[:-4]):
            diff1 = self.read_file(n[:-4]).splitlines()
        if os.path.isfile(n):
            diff2 = self.read_file(n).splitlines()
        lines, ln, c = [], 0, 0
        for a, b in itertools.izip_longest(diff1, diff2):
            ln += 1
            if a != b:
                for s1, s2 in itertools.izip_longest(str(a), str(b)):
                    c += 1
                    if s1 != s2:
                        break
                print(f"@@ -{ln},{c} +{ln},{c} @@\n")
                for line in lines[-3:]:
                    print(f"{line}")
                if a is None:
                    a = ""
                print(f"{self.red}-{self.endc}{a}")
                if b is None:
                    b = ""
                print(f"{self.green}+{self.endc}{b}")
                lines = []
                c = 0
            else:
                lines.append(a)

    def merge(self, n):
        """Merge new file into old
        """
        if os.path.isfile(n[:-4]):
            old = self.read_file(n[:-4]).splitlines()
        if os.path.isfile(n):
            new = self.read_file(n).splitlines()
        with open(n[:-4], "w") as out:
            for l1, l2 in itertools.izip_longest(old, new):
                if l1 is None:
                    l1 = ""
                if l2 is None:
                    l2 = ""
                if l1 != l2:
                    out.write(l2 + "\n")
                else:
                    out.write(l1 + "\n")
            print("The file {0} merged in file {1}".format(
                n.split("/")[-1], n[:-4].split("/")[-1]))

    def quit(self):
        raise SystemExit()



#!/usr/bin/python3
# -*- coding: utf-8 -*-

# splitting.py file is part of slpkg.

# Copyright 2014-2022 Dimitris Zlatanidis <d.zlatanidis@gmail.com>
# All rights reserved.

# Slpkg is a user-friendly package manager for Slackware installations

# https://gitlab.com/dslackw/slpkg

# Slpkg is free software: you can redistribute it and/or modify
# it under the terms of the GNU General Public License as published by
# the Free Software Foundation, either version 3 of the License, or
# (at your option) any later version.
# This program is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. See the
# GNU General Public License for more details.
# You should have received a copy of the GNU General Public License
# along with this program. If not, see <http://www.gnu.org/licenses/>.


def split_package(package):
    """
    Split package in name, version
    arch and build tag.
    """
    name = ver = arch = build = []
    split = package.split("-")

    if len(split) > 2:
        build = split[-1]
        build_a, build_b = "", ""
        build_a = build[:1]

        if build[1:2].isdigit():
            build_b = build[1:2]

        build = build_a + build_b
        arch = split[-2]
        ver = split[-3]
        name = "-".join(split[:-3])

    return [name, ver, arch, build]



#!/usr/bin/python3
# -*- coding: utf-8 -*-

# find.py file is part of slpkg.

# Copyright 2014-2022 Dimitris Zlatanidis <d.zlatanidis@gmail.com>
# All rights reserved.

# Slpkg is a user-friendly package manager for Slackware installations

# https://gitlab.com/dslackw/slpkg

# Slpkg is free software: you can redistribute it and/or modify
# it under the terms of the GNU General Public License as published by
# the Free Software Foundation, either version 3 of the License, or
# (at your option) any later version.
# This program is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. See the
# GNU General Public License for more details.
# You should have received a copy of the GNU General Public License
# along with this program. If not, see <http://www.gnu.org/licenses/>.


import os
from slpkg.blacklist import BlackList
from slpkg.splitting import split_package


def searching(find_pkg, directory):
    """Find packages
    """
    black = BlackList()
    if os.path.isdir(directory):
        installed = os.listdir(directory)
        blacklist = list(black.get_black())
        if os.path.exists(directory):
            for pkg in installed:
                if (not pkg.startswith(".") and pkg.startswith(find_pkg) and
                        split_package(pkg)[0] not in blacklist):
                    yield pkg


def find_package(pkg, path):
    """Generator allias
    """
    return list(searching(pkg, path))


#!/usr/bin/python3
# -*- coding: utf-8 -*-

# blacklist.py file is part of slpkg.

# Copyright 2014-2022 Dimitris Zlatanidis <d.zlatanidis@gmail.com>
# All rights reserved.

# Slpkg is a user-friendly package manager for Slackware installations

# https://gitlab.com/dslackw/slpkg

# Slpkg is free software: you can redistribute it and/or modify
# it under the terms of the GNU General Public License as published by
# the Free Software Foundation, either version 3 of the License, or
# (at your option) any later version.
# This program is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. See the
# GNU General Public License for more details.
# You should have received a copy of the GNU General Public License
# along with this program. If not, see <http://www.gnu.org/licenses/>.


import os

from slpkg.utils import Utils
from slpkg.splitting import split_package
from slpkg.__metadata__ import MetaData as _meta_


class BlackList(Utils):
    """Blacklist class to add, remove or listed packages
    in blacklist file."""
    def __init__(self):
        self.green = _meta_.color["GREEN"]
        self.red = _meta_.color["RED"]
        self.endc = _meta_.color["ENDC"]
        self.blackfile = "/etc/slpkg/blacklist"
        self.black_conf = ""
        if os.path.isfile(self.blackfile):
            self.black_conf = self.read_file(self.blackfile)

    def get_black(self):
        """Return blacklist packages from /etc/slpkg/blacklist
        configuration file."""
        blacklist = list(self.black_filter())
        installed = os.listdir("/var/log/packages/")

        for black in blacklist:
            if black.endswith("*"):
                for inst in installed:
                    if inst.startswith(black[:-1]):
                        yield split_package(inst)[0]
            else:
                yield black

    def black_filter(self):
        """Return all the installed files that start
        by the name*
        """
        for read in self.black_conf.splitlines():
            read = read.lstrip()
            if not read.startswith("#"):
                yield read.replace("\n", "")

    def black_listed(self):
        """Print blacklist packages
        """
        print("Packages in the blacklist:")
        for black in list(self.black_filter()):
            if black:
                print(f"{self.green}{black}{self.endc}")
        print()

    def black_add(self, pkgs):
        """Add blacklist packages if not exist
        """
        blacklist = list(self.black_filter())
        pkgs = set(pkgs)
        print("Add packages in the blacklist:")
        with open(self.blackfile, "a") as black_conf:
            for pkg in pkgs:
                if pkg not in blacklist:
                    print(f"{self.green}{pkg}{self.endc}")
                    black_conf.write(pkg + "\n")
        print()

    def black_remove(self, pkgs):
        """Remove packages from blacklist
        """
        print("Remove packages from the blacklist:")
        with open(self.blackfile, "w") as remove:
            for line in self.black_conf.splitlines():
                if line not in pkgs:
                    remove.write(line + "\n")
                else:
                    print(f"{self.red}{line}{self.endc}")
        print()



#!/usr/bin/python3
# -*- coding: utf-8 -*-

# manager.py file is part of slpkg.

# Copyright 2014-2022 Dimitris Zlatanidis <d.zlatanidis@gmail.com>
# All rights reserved.

# Slpkg is a user-friendly package manager for Slackware installations

# https://gitlab.com/dslackw/slpkg

# Slpkg is free software: you can redistribute it and/or modify
# it under the terms of the GNU General Public License as published by
# the Free Software Foundation, either version 3 of the License, or
# (at your option) any later version.
# This program is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. See the
# GNU General Public License for more details.
# You should have received a copy of the GNU General Public License
# along with this program. If not, see <http://www.gnu.org/licenses/>.


import os
import re
import subprocess


from slpkg.pkg.find import find_package
from slpkg.pkg.installed import GetFromInstalled

from slpkg.utils import Utils
from slpkg.messages import Msg
from slpkg.dialog_box import DialogUtil
from slpkg.splitting import split_package
from slpkg.__metadata__ import MetaData as _meta_

from slpkg.slack.slackware_repo import slackware_repository


class PackageManager(Utils):
    """Package manager class for install, upgrade,
    reinstall, remove, find and display packages"""
    def __init__(self, binary):
        self.binary = binary
        self.meta = _meta_
        self.green = _meta_.color["GREEN"]
        self.red = _meta_.color["RED"]
        self.yellow = _meta_.color["YELLOW"]
        self.cyan = _meta_.color["CYAN"]
        self.grey = _meta_.color["GREY"]
        self.endc = _meta_.color["ENDC"]
        self.msg = Msg()
        self.skip = []
        self.size = 0
        self.file_size = 0
        self.unit = "Kb"

    def install(self, flag):
        """Install Slackware binary packages
        """
        for pkg in self.binary:
            try:
                subprocess.call(f"installpkg {flag} {pkg}", shell=True)
                check = pkg[:-4].split("/")[-1]
                if os.path.isfile(self.meta.pkg_path + check):
                    print("Completed!\n")
                else:
                    raise SystemExit(1)
            except subprocess.CalledProcessError:
                self._not_found("Can't install", self.binary, pkg)
                raise SystemExit(1)

    def upgrade(self, flag):
        """Upgrade Slackware binary packages with new
        """
        for pkg in self.binary:
            try:
                subprocess.call(f"upgradepkg {flag} {pkg}", shell=True)
                check = pkg[:-4].split("/")[-1]
                if os.path.isfile(self.meta.pkg_path + check):
                    print("Completed!\n")
                else:
                    raise SystemExit(1)
            except subprocess.CalledProcessError:
                self._not_found("Can't upgrade", self.binary, pkg)
                raise SystemExit(1)

    def _not_found(self, message, binary, pkg):
        if len(binary) > 1:
            bol = eol = ""
        else:
            bol = eol = "\n"
        self.msg.pkg_not_found(bol, pkg, message, eol)

    def remove(self, flag, extra):
        """Remove Slackware binary packages
        """
        self.flag = flag
        self.extra = extra
        self.dep_path = self.meta.log_path + "dep/"
        dependencies, rmv_list = [], []
        self.removed = self._view_removed()
        if not self.removed:
            print()   # new line at end
        else:
            msg = "package"
            if len(self.removed) > 1:
                msg = msg + "s"
            try:
                if self.meta.default_answer in ["y", "Y"]:
                    remove_pkg = self.meta.default_answer
                else:
                    remove_pkg = input(
                        "\nAre you sure to remove {0} {1} [y/N]? ".format(
                            str(len(self.removed)), msg))
            except EOFError:
                raise SystemExit(1)
            if remove_pkg in ["y", "Y"]:
                self._check_if_used(self.binary)
                for rmv in self.removed:
                    '''
                    If package build and install with "slpkg -s sbo <package>"
                    then look log file for dependencies in /var/log/slpkg/dep,
                    read and remove all else remove only the package.
                    '''
                    if (os.path.isfile(self.dep_path + rmv) and
                            self.meta.del_deps in ["on", "ON"] or
                            os.path.isfile(self.dep_path + rmv) and
                            "--deps" in self.extra):
                        dependencies = self._view_deps(self.dep_path, rmv)
                        if dependencies and self._rmv_deps_answer() in ["y",
                                                                        "Y"]:
                            rmv_list += self._rmv_deps(dependencies, rmv)
                        else:
                            rmv_list += self._rmv_pkg(rmv)
                    else:
                        rmv_list += self._rmv_pkg(rmv)
                # Prints all removed packages
                self._reference_rmvs(rmv_list)

    def _rmv_deps_answer(self):
        """Remove dependencies answer
        """
        if self.meta.remove_deps_answer in ["y", "Y"]:
            remove_dep = self.meta.remove_deps_answer
        else:
            try:
                remove_dep = input(
                    "\nRemove dependencies (maybe used by "
                    "other packages) [y/N]? ")
                print()
            except EOFError:
                raise SystemExit("\n")
        return remove_dep

    def _get_removed(self):
        """Manage removed packages by extra options
        """
        extra = self.extra
        removed, packages, pkg = [], [], ""
        if "--tag" in extra:
            for pkg in find_package("", self.meta.pkg_path):
                for tag in self.binary:
                    if pkg.endswith(tag):
                        removed.append(split_package(pkg)[0])
                        packages.append(pkg)
            pkg = ""
            extra = ""
        elif "--third-party" in extra:
            slack_packages, slack_names = slackware_repository()
            slpkg_pkg = self.meta.__all__ + "-" + self.meta.__version__
            binary = self.binary
            for pkg in find_package("", self.meta.pkg_path):
                slack_name = split_package(pkg)[0]
                for tag in binary:
                    if (slack_name not in slack_names and
                            slpkg_pkg not in pkg and tag in pkg):
                        removed.append(split_package(pkg)[0])
                        packages.append(pkg)
            pkg = ""
            extra = ""
        else:
            for pkg in self.binary:
                name = GetFromInstalled(pkg).name()
                ver = GetFromInstalled(pkg).version()
                package = find_package(f"{name}-{ver}-", self.meta.pkg_path)
                if pkg and name == pkg:
                    removed.append(pkg)
                    packages.append(package[0])
        if not removed:
            self.msg.pkg_not_found("", pkg, "Can't remove", "\n")
            raise SystemExit(1)
        return removed, packages

    def _view_removed(self):
        """View packages before removed
        """
        print("Packages with name matching [ {0}{1}{2} ]\n".format(
            self.cyan, ", ".join(self.binary), self.endc))
        removed, packages = self._get_removed()
        if packages and "--checklist" in self.extra:
            removed = []
            text = "Press 'spacebar' to unchoose packages from the remove"
            backtitle = f"{self.meta.__all__} {self.meta.__version__}"
            status = True
            pkgs = DialogUtil(packages, text, " Remove ", backtitle,
                              status).checklist()
            if pkgs:
                for rmv in pkgs:
                    removed.append(split_package(rmv)[0])
                self.meta.default_answer = "y"
        else:
            for rmv, pkg in zip(removed, packages):
                self._sizes(pkg)
                print(f"[ {self.red}delete{self.endc} ] --> "
                      f"[ {self.file_size} ] - {pkg}")
            self._calc_sizes()
            self._remove_summary()
        if "--third-party" in self.extra:
            print()
            self.msg.template(78)
            print(f"| {' ' * 27}{self.red}*** WARNING ***{self.endc}")
            print("| Before you use third-party option, be sure you have"
                  " updated the packages \n| lists. Run the command"
                  " 'slpkg update' and 'slpkg -c slack --upgrade'")
            self.msg.template(78)
        return removed

    def _calc_sizes(self):
        """Package size calculation
        """
        if self.size > 1024:
            self.unit = "Mb"
            self.size = (self.size / 1024)
        if self.size > 1024:
            self.unit = "Gb"
            self.size = (self.size / 1024)

    def _remove_summary(self):
        """Removed packge size summary
        """
        if self.size > 0:
            print("\nRemoved summary")
            print("=" * 79)
            print("{0}Size of removed packages {1} {2}.{3}".format(
                self.grey, round(self.size, 2), self.unit, self.endc))

    def _view_deps(self, path, package):
        """View dependencies before remove
        """
        self.size = 0
        packages = []
        dependencies = (self.read_file(path + package)).splitlines()
        for dep in dependencies:
            if GetFromInstalled(dep).name():
                ver = GetFromInstalled(dep).version()
                packages.append(f"{dep}-{ver}")
            else:
                dependencies.remove(dep)
        if packages:
            if "--checklist" in self.extra:
                deps, dependencies = [], []
                text = "Found dependencies for the package {0}".format(package)
                backtitle = f"{self.meta.__all__} {self.meta.__version__}"
                status = True
                deps = DialogUtil(packages, text, " Remove ", backtitle,
                                  status).checklist()
                for d in deps:
                    dependencies.append("-".join(d.split("-")[:-1]))
                self.meta.remove_deps_answer = "y"
            else:
                print()   # new line at start
                self.msg.template(78)
                print(f"| Found dependencies for the package {package}:")
                self.msg.template(78)
                for pkg in packages:
                    find = find_package(f"{pkg}-", self.meta.pkg_path)
                    self._sizes(find[0])
                    print(f"| {self.red}{pkg}{self.endc}")
                self.msg.template(78)
                self._calc_sizes()
                print("| {0}Size of removed dependencies {1} {2}{3}".format(
                    self.grey, round(self.size, 2), self.unit, self.endc))
                self.msg.template(78)
        return dependencies

    def _removepkg(self, package):
        """removepkg Slackware command
        """
        try:
            subprocess.call(f"removepkg {self.flag} {package}", shell=True)
            if os.path.isfile(self.dep_path + package):
                os.remove(self.dep_path + package)  # remove log
        except subprocess.CalledProcessError as er:
            raise SystemExit(er)

    def _rmv_deps(self, dependencies, package):
        """Remove dependencies
        """
        removes = []
        dependencies.append(package)
        self._check_if_used(dependencies)
        for dep in dependencies:
            if dep not in self.skip and GetFromInstalled(dep).name():
                ver = GetFromInstalled(dep).version()
                removes.append(f"{dep}-{ver}")
                self._removepkg(dep)
        return removes

    def _rmv_pkg(self, package):
        """Remove one signle package
        """
        removes = []
        if GetFromInstalled(package).name() and package not in self.skip:
            ver = GetFromInstalled(package).version()
            removes.append(f"{package}-{ver}")
            self._removepkg(package)
        return removes

    def _skip_remove(self):
        """Skip packages from remove
        """
        if "--checklist" not in self.extra:
            self.msg.template(78)
            print("| Insert packages to exception remove:")
            self.msg.template(78)
            try:
                self.skip = input(" > ").split()
            except EOFError:
                raise SystemExit("\n")
        for s in self.skip:
            if s in self.removed:
                self.removed.remove(s)

    def _check_if_used(self, removes):
        """Check package if dependencies for another package
        before removed"""
        if "--check-deps" in self.extra:
            package, dependency, pkg_dep = [], [], []
            for pkg in find_package("", self.dep_path):
                deps = self.read_file(self.dep_path + pkg)
                for rmv in removes:
                    if GetFromInstalled(rmv).name() and rmv in deps.split():
                        pkg_dep.append(f"{rmv} is dependency of the "
                                       f"package --> {pkg}")
                        package.append(pkg)
                        dependency.append(rmv)
            if package:
                if "--checklist" in self.extra:
                    text = ("Press 'spacebar' to choose packages for the"
                            " remove exception")
                    backtitle = f"{self.meta.__all__} {self.meta.__version__}"
                    status = False
                    choose = DialogUtil(pkg_dep, text, " !!! WARNING !!! ",
                                        backtitle, status).checklist()
                    for pkg in choose:
                        self.skip.append(pkg.split()[0])
                else:
                    self.msg.template(78)
                    print("| {0}{1}{2}".format(
                        self.red, " " * 30 + "!!! WARNING !!!", self.endc))
                    self.msg.template(78)
                    for p, d in zip(package, dependency):
                        print(f"| {self.yellow}{d}{self.endc} is dependency "
                              f"of the package --> {self.green}{p}{self.endc}")
                    self.msg.template(78)
                    self._skip_remove()

    def _reference_rmvs(self, removes):
        """Prints all removed packages
        """
        print()
        self.msg.template(78)
        msg_pkg = "package"
        if len(removes) > 1:
            msg_pkg = "packages"
        print(f"| Total {len(removes)} {msg_pkg} removed")
        self.msg.template(78)
        for pkg in removes:
            if not GetFromInstalled(pkg).name():
                print(f"| Package {pkg} removed")
            else:
                print(f"| Package {pkg} not found")
        self.msg.template(78)
        print()   # new line at end

    def find(self, flag):
        """Find installed Slackware packages
        """
        matching, packages = 0, []
        pkg_cache, match_cache = "", ""
        slack_packages, slack_names = slackware_repository()
        print("Packages with matching name [ {0}{1}{2} ]\n".format(
            self.cyan, ", ".join(self.binary), self.endc))
        for pkg in self.binary:
            for match in find_package("", self.meta.pkg_path):
                pkg_cache = pkg
                match_cache = match
                split_name = split_package(match)[0]
                if "--case-ins" in flag:
                    pkg_cache = pkg.lower()
                    match_cache = match.lower()
                if ("--third-party" in flag and not self.binary and
                        split_name not in slack_names):
                    packages.append(match)
                if ("--third-party" in flag and pkg_cache in match_cache and
                        split_name not in slack_names):
                    packages.append(match)
                if pkg_cache in match_cache and not flag:
                    packages.append(match)
                if ("--case-ins" in flag and "--third-party" not in flag and
                        pkg_cache in match_cache):
                    packages.append(match)
        for pkgs in packages:
            matching += 1
            self._sizes(pkgs)
            print(f"[ {self.green}installed{self.endc} ] "
                  f"[ {self.file_size} ] - {pkgs}")
        if matching == 0:
            message = "Can't find"
            self.msg.pkg_not_found("", ", ".join(self.binary), message, "\n")
            raise SystemExit(1)
        else:
            self._calc_sizes()
            print("\nFound summary")
            print("=" * 79)
            print("{0}Total found {1} matching packages.{2}".format(
                self.grey, matching, self.endc))
            print("{0}Size of installed packages {1} {2}.{3}\n".format(
                self.grey, round(self.size, 2), self.unit, self.endc))

    def _sizes(self, package):
        """Package size summary
        """
        data = self.read_file(self.meta.pkg_path + package)
        for line in data.splitlines():
            if line.startswith("UNCOMPRESSED PACKAGE SIZE:"):
                digit = float((''.join(re.findall(
                    r"[-+]?\d+[\.]?\d*[eE]?[-+]?\d*", line[26:]))))
                self.file_size = line[26:].strip()
                if "M" in line[26:]:
                    self.size += digit * 1024
                else:
                    self.size += digit
                    break

    def display(self):
        """Print the Slackware packages contents
        """
        for pkg in self.binary:
            name = GetFromInstalled(pkg).name()
            ver = GetFromInstalled(pkg).version()
            find = find_package(f"{name}-{ver}-", self.meta.pkg_path)
            if find:
                package = self.read_file(
                    self.meta.pkg_path + "".join(find))
                print(package)
            else:
                message = "Can't dislpay"
                if len(self.binary) > 1:
                    bol = eol = ""
                else:
                    bol = eol = "\n"
                self.msg.pkg_not_found(bol, pkg, message, eol)
                raise SystemExit(1)

    def package_list(self, repo, name, INDEX, installed):
        """List with the installed packages
        """
        tty_size = os.popen("stty size", "r").read().split()
        row = int(tty_size[0]) - 2
        try:
            all_installed_names = []
            index, page, pkg_list = 0, row, []
            r = self.list_lib(repo)
            pkg_list = self.list_greps(repo, r)[0]
            all_installed_names = self.list_of_installed(repo, name)
            print()
            for pkg in sorted(pkg_list):
                pkg = self._splitting_packages(pkg, repo, name)
                if installed and repo:
                    if pkg in all_installed_names:
                        pkg = f"{self.green}{pkg}{self.endc}"

                if INDEX:
                    index += 1
                    pkg = self.list_color_tag(pkg)
                    print(f"{self.grey}{index}:{self.endc} {pkg}")
                    if index == page:
                        read = input(f"\nPress {self.cyan}Enter{self.endc} to "
                                     f"continue... ")
                        if read in ["Q", "q"]:
                            break
                        print()   # new line after page
                        page += row
                else:
                    print(pkg)
            print()   # new line at end
        except (EOFError, KeyboardInterrupt, BrokenPipeError, IOError):
            raise SystemExit(1)

    def _splitting_packages(self, pkg, repo, name):
        """Return package name from repositories
        """
        if name and repo != "sbo":
            pkg = split_package(pkg)[0]
        elif not name and repo != "sbo":
            pkg = pkg[:-4]
        return pkg

    def list_greps(self, repo, packages):
        """Grep packages
        """
        pkg_list, pkg_size = [], []
        for line in packages.splitlines():
            if repo == "sbo":
                if line.startswith("SLACKBUILD NAME: "):
                    pkg_list.append(line[17:].strip())
                    pkg_size.append("0 K")
            else:
                if line.startswith("PACKAGE NAME: "):
                    pkg_list.append(line[15:].strip())
                if line.startswith("PACKAGE SIZE (compressed): "):
                    pkg_size.append(line[26:].strip())
        if repo == "alien" or repo == "ktown":
            return alien_filter(pkg_list, pkg_size)
        return pkg_list, pkg_size

    def list_lib(self, repo):
        """Return package lists
        """
        packages = ""
        if repo == "sbo":
            if (os.path.isfile(
                    self.meta.lib_path + "sbo_repo/SLACKBUILDS.TXT")):
                packages = self.read_file(f"{self.meta.lib_path}"
                                          "sbo_repo/SLACKBUILDS.TXT")
        else:
            if (os.path.isfile(
                    self.meta.lib_path + f"{repo}_repo/PACKAGES.TXT")):
                packages = self.read_file(f"{self.meta.lib_path}"
                                          f"{repo}_repo/PACKAGES.TXT")
        return packages

    def list_color_tag(self, pkg):
        """Tag with color installed packages
        """
        name = GetFromInstalled(pkg).name()
        find = f"{name}-"
        if pkg.endswith(".txz") or pkg.endswith(".tgz"):
            find = pkg[:-4]
        if find_package(find, self.meta.pkg_path):
            pkg = f"{self.green}{pkg}{self.endc}"
        return pkg

    def list_of_installed(self, repo, name):
        """Return installed packages
        """
        all_installed_names = []
        all_installed_packages = find_package("", self.meta.pkg_path)
        for inst in all_installed_packages:
            if repo == "sbo" and inst.endswith("_SBo"):
                name = split_package(inst)[0]
                all_installed_names.append(name)
            else:
                if name:
                    all_installed_names.append(split_package(inst)[0])
                else:
                    all_installed_names.append(inst)
        return all_installed_names


def alien_filter(packages, sizes):
    """This filter avoid list double packages from
    alien repository
    """
    cache, npkg, nsize = [], [], []
    for p, s in zip(packages, sizes):
        name = split_package(p)[0]
        if name not in cache:
            cache.append(name)
            npkg.append(p)
            nsize.append(s)
    return npkg, nsize



#!/usr/bin/python3
# -*- coding: utf-8 -*-

# load.py file is part of slpkg.

# Copyright 2014-2022 Dimitris Zlatanidis <d.zlatanidis@gmail.com>
# All rights reserved.

# Slpkg is a user-friendly package manager for Slackware installations

# https://gitlab.com/dslackw/slpkg

# Slpkg is free software: you can redistribute it and/or modify
# it under the terms of the GNU General Public License as published by
# the Free Software Foundation, either version 3 of the License, or
# (at your option) any later version.
# This program is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. See the
# GNU General Public License for more details.
# You should have received a copy of the GNU General Public License
# along with this program. If not, see <http://www.gnu.org/licenses/>.


import os

from slpkg.utils import Utils
from slpkg.splitting import split_package
from slpkg.__metadata__ import MetaData as _meta_

from slpkg.pkg.find import find_package


def library(repo):
    """Load packages from slpkg library and from local
    """
    utils = Utils()
    pkg_list, packages = [], ""
    if repo == "sbo":
        if (os.path.isfile(
                f"{_meta_.lib_path}{repo}_repo/SLACKBUILDS.TXT")):
            packages = utils.read_file(
                f"{_meta_.lib_path}{repo}_repo/SLACKBUILDS.TXT")
    else:
        if (os.path.isfile(
                f"{_meta_.lib_path}{repo}_repo/PACKAGES.TXT")):
            packages = utils.read_file(
                f"{_meta_.lib_path}{repo}_repo/PACKAGES.TXT")
    for line in packages.splitlines():
        if repo == "sbo":
            if line.startswith("SLACKBUILD NAME: "):
                pkg_list.append(line[17:].strip())
        elif "local" not in repo:
            if line.startswith("PACKAGE NAME: "):
                pkg_list.append(line[15:].strip())
    if repo == "local":
        pkg_list = find_package("", _meta_.pkg_path)
    return pkg_list


class Regex:
    """Grap packages with simple regex using asterisk *
       with options: starts with string*
                     ends with *string
                     include *string*
    """
    def __init__(self, pkgs):
        self.pkgs = pkgs

    def get(self):
        lib, data = [], []
        for pkg in self.pkgs.split(","):
            pr = pkg.split(":")     # priotity by repository
            data = library(pr[0])   # load data
            if len(pr) > 1:
                for d in data:
                    if pr[1].startswith("*") and pr[1].endswith("*"):
                        if pr[1][1:-1] in d:
                            lib.append(self.add(pr[0], d))
                    elif pr[1].endswith("*"):
                        if d.startswith(pr[1][:-1]):
                            lib.append(self.add(pr[0], d))
                    elif pr[1].startswith("*"):
                        if d.endswith(pr[1][1:]):
                            lib.append(self.add(pr[0], d))
                    else:
                        lib.append(self.add(pr[0], d))
            else:
                lib += pkg.split()
        return lib

    def add(self, repo, pkg):
        """Split packages by repository
        """
        if repo == "sbo":
            return pkg
        else:
            return split_package(pkg)[0]



#!/usr/bin/python3
# -*- coding: utf-8 -*-

# auto_pkg.py file is part of slpkg.

# Copyright 2014-2022 Dimitris Zlatanidis <d.zlatanidis@gmail.com>
# All rights reserved.

# Slpkg is a user-friendly package manager for Slackware installations

# https://gitlab.com/dslackw/slpkg

# Slpkg is free software: you can redistribute it and/or modify
# it under the terms of the GNU General Public License as published by
# the Free Software Foundation, either version 3 of the License, or
# (at your option) any later version.
# This program is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. See the
# GNU General Public License for more details.
# You should have received a copy of the GNU General Public License
# along with this program. If not, see <http://www.gnu.org/licenses/>.


from slpkg.messages import Msg
from slpkg.__metadata__ import MetaData as _meta_

from slpkg.pkg.manager import PackageManager


class Auto:
    """Select Slackware command to install packages
    """
    def __init__(self, packages):
        self.packages = packages
        self.green = _meta_.color["GREEN"]
        self.red = _meta_.color["RED"]
        self.cyan = _meta_.color["CYAN"]
        self.endc = _meta_.color["ENDC"]
        self.msg = Msg()
        self.commands = {
            "i": "installpkg",
            "u": "upgradepkg --install-new",
            "r": "upgradepkg --reinstall"
        }

    def select(self):
        """Select Slackware command
        """
        print("\nDetected Slackware binary package for installation:\n")
        for pkg in self.packages:
            print(" " + pkg.split("/")[-1])
        print()
        self.msg.template(78)
        print("| Choose a Slackware command:")
        self.msg.template(78)
        for com in sorted(self.commands):
            print(f"| {self.red}{com}{self.endc}) {self.green}"
                  f"{self.commands[com]}{self.endc}")
        self.msg.template(78)
        try:
            self.choice = input(" > ")
        except EOFError:
            raise SystemExit("\n")
        if self.choice in self.commands.keys():
            print(f"   \x1b[1A{self.cyan}{self.commands[self.choice]}"
                  f"{self.endc}", end="\n\n")
            print(end="", flush=True)
        self.execute()

    def execute(self):
        """Execute Slackware command
        """
        if self.choice in self.commands.keys():
            if self.choice == "i":
                PackageManager(self.packages).install("")
            elif self.choice in ["u", "r"]:
                PackageManager(self.packages).upgrade(
                    self.commands[self.choice][11:])



import unittest
from slpkg.file_size import FileSize


class TestFileSize(unittest.TestCase):

    def test_FileSize(self):
        """Testing the remote and local servers
        """
        url = "https://mirrors.slackware.com/slackware/slackware64-14.2/ChangeLog.txt"
        lc = "test_units.py"
        fs1 = FileSize(url)
        fs2 = FileSize(lc)
        self.assertIsNotNone(fs1.server())
        self.assertIsNotNone(fs2.local())


if __name__ == "__main__":
    unittest.main()



#!/usr/bin/python3
# -*- coding: utf-8 -*

# file_size.py file is part of slpkg.

# Copyright 2014-2022 Dimitris Zlatanidis <d.zlatanidis@gmail.com>
# All rights reserved.

# Slpkg is a user-friendly package manager for Slackware installations

# https://gitlab.com/dslackw/slpkg

# Slpkg is free software: you can redistribute it and/or modify
# it under the terms of the GNU General Public License as published by
# the Free Software Foundation, either version 3 of the License, or
# (at your option) any later version.
# This program is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. See the
# GNU General Public License for more details.
# You should have received a copy of the GNU General Public License
# along with this program. If not, see <http://www.gnu.org/licenses/>.


import os
import urllib3

from slpkg.__metadata__ import MetaData as _meta_


class FileSize:
    """Check local or remote file size
    """
    def __init__(self, registry):
        self.meta = _meta_
        self.registry = registry
        if self.meta.http_proxy:
            self.http = urllib3.ProxyManager(self.meta.http_proxy)
        else:
            self.http = urllib3.PoolManager()

    def server(self):
        """Returns the size of remote files
        """
        try:
            r = self.http.request("GET", self.registry)
            return int(r.headers["Content-Length"])
        except urllib3.exceptions.NewConnectionError:
            return " "

    def local(self):
        """Returns the size of local files
        """
        return os.path.getsize(self.registry)



import unittest
from slpkg.utils import Utils


class TestUtils(unittest.TestCase):

    def setUp(self):
        self.utils = Utils()

    def test_dimensional_list(self):
        """Testing dimesional list util
        """
        lists = [[1, 2, 3, 4, 5]]
        self.assertListEqual([1, 2, 3, 4, 5],
                             self.utils.dimensional_list(lists))

    def test_remove_dbs(self):
        """Testing removing doubles item from list
        """
        lists = [1, 2, 3, 3, 4, 5, 2, 1]
        self.assertListEqual([1, 2, 3, 4, 5], self.utils.remove_dbs(lists))

    def test_case_sensitive(self):
        """Testing case sensitive
        """
        lists = ['Vlc', 'OpenOffice', 'APScheduler']
        dictionary = {'vlc': 'Vlc',
                      'openoffice': 'OpenOffice',
                      'apscheduler': 'APScheduler'}
        self.assertDictEqual(dictionary, self.utils.case_sensitive(lists))


if __name__ == "__main__":
    unittest.main()



#!/usr/bin/python3
# -*- coding: utf-8 -*-

# clean.py file is part of slpkg.

# Copyright 2014-2022 Dimitris Zlatanidis <d.zlatanidis@gmail.com>
# All rights reserved.

# Slpkg is a user-friendly package manager for Slackware installations

# https://gitlab.com/dslackw/slpkg

# Slpkg is free software: you can redistribute it and/or modify
# it under the terms of the GNU General Public License as published by
# the Free Software Foundation, either version 3 of the License, or
# (at your option) any later version.
# This program is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. See the
# GNU General Public License for more details.
# You should have received a copy of the GNU General Public License
# along with this program. If not, see <http://www.gnu.org/licenses/>.


import os
import shutil

from slpkg.__metadata__ import MetaData as _meta_


def clean_tmp():
    """Delete packages and sources from tmp/ directory
    """
    tmps = [_meta_.tmp_path,  # /tmp/slpkg/
            _meta_.build_path,  # /tmp/slpkg/build/
            _meta_.slpkg_tmp_packages,  # /tmp/slpkg/packages/
            _meta_.slpkg_tmp_patches  # /tmp/slpkg/patches/
            ]
    # Delete a whole slpkg folder from the tmp directory
    if os.path.exists(tmps[0]):
        shutil.rmtree(tmps[0])
        print(f"All packages and sources were deleted from: {tmps[0]}")
    # Recreate the paths again
    if not os.path.exists(tmps[0]):
        for tmp in tmps:
            print(f"Created directory: {tmp}")
            os.mkdir(tmp)
        print("Done!")


#!/usr/bin/python3
# -*- coding: utf-8 -*-

# pkg_find.py file is part of slpkg.

# Copyright 2014-2022 Dimitris Zlatanidis <d.zlatanidis@gmail.com>
# All rights reserved.

# Slpkg is a user-friendly package manager for Slackware installations

# https://gitlab.com/dslackw/slpkg

# Slpkg is free software: you can redistribute it and/or modify
# it under the terms of the GNU General Public License as published by
# the Free Software Foundation, either version 3 of the License, or
# (at your option) any later version.
# This program is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. See the
# GNU General Public License for more details.
# You should have received a copy of the GNU General Public License
# along with this program. If not, see <http://www.gnu.org/licenses/>.


from slpkg.messages import Msg
from slpkg.sbo.greps import SBoGrep
from slpkg.pkg.manager import PackageManager
from slpkg.__metadata__ import MetaData as _meta_


class FindFromRepos:
    """Find packages from all enabled repositories
    """
    def __init__(self):
        self.cache = ""
        self.p_cache = ""
        self.find_cache = ""
        self.count_pkg = 0
        self.count_repo = 0
        self.meta = _meta_
        self.msg = Msg()
        self.cyan = self.meta.color["CYAN"]
        self.grey = self.meta.color["GREY"]
        self.endc = self.meta.color["ENDC"]

    def find(self, pkg, flag):
        """Start to find packages and print
        """
        print(f"Packages with name matching [ {self.cyan}"
              f"{', '.join(pkg)}{self.endc} ]\n")
        self.msg.template(78)
        print(f"| Repository Package {' ' * 54}Size")
        self.msg.template(78)
        for repo in _meta_.repositories:
            PACKAGES_TXT = PackageManager(pkg).list_lib(repo)
            packages, sizes = PackageManager(pkg).list_greps(repo,
                                                             PACKAGES_TXT)
            for find, size in zip(packages, sizes):
                for p in pkg:
                    if "--case-ins" in flag:
                        self.p_cache = p.lower()
                        self.find_cache = find.lower()
                    else:
                        self.p_cache = p
                        self.find_cache = find
                    if self.p_cache in self.find_cache:
                        if self.cache != repo:
                            self.count_repo += 1
                        self.cache = repo
                        self.count_pkg += 1
                        ver = self.sbo_version(repo, find)
                        print(f"  {self.cyan}{repo}{self.endc}"
                              f"{' ' * (11 - len(repo))}{find + ver} "
                              f"{' ' * (54 -len(find + ver))}{size:>11}")
        print("\nFound summary")
        print("=" * 79)
        print(f"{self.grey}Total found {self.count_pkg} packages in "
              f"{self.count_repo} repositories.{self.endc}\n")

    def sbo_version(self, repo, find):
        """
        Add version to SBo packages
        """
        ver = ""
        if repo == "sbo":
            ver = f"-{SBoGrep(find).version()}"
        return ver



#!/usr/bin/python3
# -*- coding: utf-8 -*-

# search.py file is part of slpkg.

# Copyright 2014-2022 Dimitris Zlatanidis <d.zlatanidis@gmail.com>
# All rights reserved.

# Slpkg is a user-friendly package manager for Slackware installations

# https://gitlab.com/dslackw/slpkg

# Slpkg is free software: you can redistribute it and/or modify
# it under the terms of the GNU General Public License as published by
# the Free Software Foundation, either version 3 of the License, or
# (at your option) any later version.
# This program is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. See the
# GNU General Public License for more details.
# You should have received a copy of the GNU General Public License
# along with this program. If not, see <http://www.gnu.org/licenses/>.


from slpkg.utils import Utils
from slpkg.blacklist import BlackList
from slpkg.__metadata__ import MetaData as _meta_


def search_pkg(name, repo):
    """Search if package exists in PACKAGES.TXT file
    and return the name.
    """
    utils = Utils()
    black = BlackList()
    text = utils.read_file(_meta_.lib_path + f"{repo}_repo/PACKAGES.TXT")
    PACKAGES_TXT = list(utils.package_name(text))
    blacklist = list(black.get_black())
    if name in PACKAGES_TXT and name not in blacklist:
        return name


#!/usr/bin/python3
# -*- coding: utf-8 -*-

# patches.py file is part of slpkg.

# Copyright 2014-2022 Dimitris Zlatanidis <d.zlatanidis@gmail.com>
# All rights reserved.

# Slpkg is a user-friendly package manager for Slackware installations

# https://gitlab.com/dslackw/slpkg

# Slpkg is free software: you can redistribute it and/or modify
# it under the terms of the GNU General Public License as published by
# the Free Software Foundation, either version 3 of the License, or
# (at your option) any later version.
# This program is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. See the
# GNU General Public License for more details.
# You should have received a copy of the GNU General Public License
# along with this program. If not, see <http://www.gnu.org/licenses/>.


import os
import shutil
import subprocess
from pkg_resources import parse_version


from slpkg.init import Update
from slpkg.utils import Utils
from slpkg.sizes import units
from slpkg.messages import Msg
from slpkg.url_read import URL
from slpkg.checksum import check_md5
from slpkg.blacklist import BlackList
from slpkg.downloader import Download
from slpkg.remove import delete_package
from slpkg.grep_md5 import pkg_checksum
from slpkg.dialog_box import DialogUtil
from slpkg.splitting import split_package
from slpkg.__metadata__ import MetaData as _meta_

from slpkg.pkg.find import find_package
from slpkg.pkg.manager import PackageManager
from slpkg.pkg.installed import GetFromInstalled

from slpkg.binary.greps import repo_data

from slpkg.slack.mirrors import mirrors
from slpkg.slack.slack_version import slack_ver


class Patches(BlackList, Utils):
    """Upgrade distribution from official Slackware mirrors
    """
    def __init__(self, skip, flag):
        super().__init__()
        self.skip = skip
        self.flag = flag
        self.meta = _meta_
        self.grey = _meta_.color["GREY"]
        self.yellow = _meta_.color["YELLOW"]
        self.green = _meta_.color["GREEN"]
        self.red = _meta_.color["RED"]
        self.endc = _meta_.color["ENDC"]
        self.msg = Msg()
        self.version = self.meta.slack_rel
        self.patch_path = self.meta.slpkg_tmp_patches
        self.count_added = 0
        self.count_upg = 0
        self.pkg_for_upgrade = []
        self.dwn_links = []
        self.upgrade_all = []
        self.upgraded = []
        self.installed = []
        self.comp_sum = []
        self.uncomp_sum = []
        self.msg.checking()
        if self.version == "stable":
            self.PACKAGES_TXT = URL(mirrors("PACKAGES.TXT",
                                            "patches/")).get_request()
        else:
            self.PACKAGES_TXT = URL(mirrors("PACKAGES.TXT", "")).get_request()

    def start(self):
        """Install new patches from official Slackware mirrors
        """
        self.store()
        self.msg.done()
        if self.upgrade_all:
            if "--checklist" in self.flag:
                self.dialog_checklist()
            print("\nThese packages need upgrading:\n")
            self.msg.template(78)
            print(f"| Package{' ' * 17}New Version{' ' * 8}Arch"
                  f"{' ' * 4}Build{' ' * 2}Repo{' ' * 11}Size")
            self.msg.template(78)
            print("Upgrading:")
            self.views()
            unit, size = units(self.comp_sum, self.uncomp_sum)
            print("\nInstalling summary")
            print("=" * 79)
            print(f"{self.grey}Total {self.count_upg} "
                  f"{self.msg.pkg(len(self.upgrade_all))} will be upgraded and"
                  f" {self.count_added} will be installed.")
            print(f"Need to get {size[0]} {unit[0]} of archives.")
            print(f"After this process, {size[1]} {unit[1]} of additional disk"
                  f" space will be used.{self.endc}")
            print()
            if self.msg.answer() in ["y", "Y"]:
                Download(self.patch_path, self.dwn_links,
                         repo="slack").start()
                self.upgrade_all = list(self.check_downloaded(
                    self.patch_path, self.upgrade_all))
                self.upgrade()
                self.kernel()
                if self.meta.slackpkg_log in ["on", "ON"]:
                    # update the slackpkg ChanheLog.txt file
                    self.slackpkg_update()
                self.msg.reference(self.installed, self.upgraded)
                # delete the downloaded packages
                delete_package(self.patch_path, self.upgrade_all)
                # update the packages lists
                self.update_lists()
        else:
            slack_arch = ""
            if self.meta.arch == "x86_64":
                slack_arch = "64"
            print(f"\nSlackware{slack_arch} '{self.version}' v{slack_ver()}"
                  f" distribution is up to date!\n")

    def store(self):
        """Store and return packages for upgrading
        """
        data = repo_data(self.PACKAGES_TXT, "slack", self.flag)
        black = list(self.get_black())
        for name, loc, comp, uncomp in zip(data[0], data[1], data[2], data[3]):
            repo_pkg_name = split_package(name)[0]
            if self.meta.only_installed in ["on", "ON"]:
                pkg_name = split_package(name)[0]
                pkg_ver = split_package(name)[1]
                if (GetFromInstalled(pkg_name).name() and
                        repo_pkg_name not in black and
                        repo_pkg_name not in self.skip and
                        parse_version(pkg_ver) > parse_version(
                            GetFromInstalled(pkg_name).version())):
                    self.dwn_links.append(f"{mirrors('', '')}{loc}/{name}")
                    self.comp_sum.append(comp)
                    self.uncomp_sum.append(uncomp)
                    self.upgrade_all.append(name)
                    self.count_upg += 1
                    if not find_package(f"{repo_pkg_name}-",
                                        self.meta.pkg_path):
                        self.count_added += 1
                        self.count_upg -= 1
            elif (not os.path.isfile(self.meta.pkg_path + name[:-4]) and
                    repo_pkg_name not in black and
                    repo_pkg_name not in self.skip):
                self.dwn_links.append(f"{mirrors('', '')}{loc}/{name}")
                self.comp_sum.append(comp)
                self.uncomp_sum.append(uncomp)
                self.upgrade_all.append(name)
                self.count_upg += 1
                if not find_package(f"{repo_pkg_name}-", self.meta.pkg_path):
                    self.count_added += 1
                    self.count_upg -= 1
        return self.count_upg

    def dialog_checklist(self):
        """Create checklist to choose packages for upgrade
        """
        data = []
        for upg in self.upgrade_all:
            data.append(upg[:-4])
        text = "Press 'spacebar' to unchoose packages from upgrade"
        title = " Upgrade "
        status = True
        backtitle = f"{self.meta.__all__} {self.meta.__version__}"
        pkgs = DialogUtil(data, text, title, backtitle,
                          status).checklist()
        index = 0
        for pkg, comp, uncomp in zip(self.upgrade_all, self.comp_sum,
                                     self.uncomp_sum):
            if pkg[:-4] not in pkgs:
                self.dwn_links.pop(index)
                self.upgrade_all.pop(index)
                self.comp_sum.pop(index)
                self.uncomp_sum.pop(index)
                self.count_upg -= 1
                del comp, uncomp
                index -= 1
            index += 1
        if not self.upgrade_all:
            raise SystemExit()

    def views(self):
        """Views packages
        """
        for upg, size in sorted(zip(self.upgrade_all, self.comp_sum)):
            pkg_repo = split_package(upg[:-4])
            color = self.red
            pkg_inst = GetFromInstalled(pkg_repo[0]).name()
            if pkg_repo[0] == pkg_inst:
                color = self.yellow
            ver = GetFromInstalled(pkg_repo[0]).version()
            if ver:
                ver = f"-{ver}"
            print(f"  {color}{pkg_repo[0] + ver}{self.endc}"
                  f"{' ' * (23-len(pkg_repo[0] + ver))} {pkg_repo[1]}"
                  f"{' ' * (18-len(pkg_repo[1]))} {pkg_repo[2]}"
                  f"{' ' * (8-len(pkg_repo[2]))}{pkg_repo[3]}"
                  f"{' ' * (7-len(pkg_repo[3]))}Slack{size:>12} K")

    def upgrade(self):
        """Upgrade packages
        """
        for pkg in self.upgrade_all:
            check_md5(pkg_checksum(pkg, "slack_patches"),
                      self.patch_path + pkg)
            pkg_ver = f"{split_package(pkg)[0]}-{split_package(pkg)[1]}"
            if find_package(f"{split_package(pkg)[0]}-",
                            self.meta.pkg_path):
                print(f"[ {self.yellow}upgrading{self.endc} ] --> {pkg[:-4]}")
                PackageManager((self.patch_path + pkg).split()).upgrade(
                    "--install-new")
                self.upgraded.append(pkg_ver)
            else:
                print(f"[ {self.green}installing{self.endc} ] --> {pkg[:-4]}")
                PackageManager((self.patch_path + pkg).split()).upgrade(
                    "--install-new")
                self.installed.append(pkg_ver)

    def kernel(self):
        """Check if kernel upgraded if true
        then reinstall "lilo"
        """
        for core in self.upgrade_all:
            if "kernel" in core:
                if self.meta.default_answer in ["y", "Y"]:
                    answer = self.meta.default_answer
                else:
                    print()
                    self.msg.template(78)
                    print(f"| {self.red}*** HIGHLY recommended reinstall "
                          f"boot loader ***{self.endc}")
                    print("| L=lilo / E=elilo / G=grub")
                    self.msg.template(78)
                    try:
                        answer = input("\nThe kernel has been upgraded, "
                                       "reinstall boot loader [L/E/G]? ")
                    except EOFError:
                        raise SystemExit("\n")
                if answer in ["L"]:
                    subprocess.call("lilo", shell=True)
                    break
                elif answer in ["E"]:
                    subprocess.call("eliloconfig", shell=True)
                    break
                elif answer in ["G"]:
                    subprocess.call("grub-mkconfig -o /boot/grub/grub.cfg",
                                    shell=True)
                    break

    def slackpkg_update(self):
        """This replace slackpkg ChangeLog.txt file with new
        from Slackware official mirrors after update distribution.
        """
        NEW_ChangeLog_txt = URL(mirrors("ChangeLog.txt", "")).get_request()
        if os.path.isfile(f"{self.meta.slackpkg_lib_path}ChangeLog.txt.old"):
            os.remove(f"{self.meta.slackpkg_lib_path}ChangeLog.txt.old")
        if os.path.isfile(f"{self.meta.slackpkg_lib_path}ChangeLog.txt"):
            shutil.copy2(f"{self.meta.slackpkg_lib_path}ChangeLog.txt",
                         f"{self.meta.slackpkg_lib_path}ChangeLog.txt.old")
            os.remove(f"{self.meta.slackpkg_lib_path}ChangeLog.txt")
        with open(f"{self.meta.slackpkg_lib_path}ChangeLog.txt", "w") as log:
            log.write(NEW_ChangeLog_txt)

    def update_lists(self):
        """Update packages list and ChangeLog.txt file after
        upgrade distribution
        """
        print(f"{self.green}Update the package lists ?{self.endc}")
        print("=" * 79)
        if self.msg.answer() in ["y", "Y"]:
            Update().run(["slack"])



#!/usr/bin/python3
# -*- coding: utf-8 -*-

# health.py file is part of slpkg.

# Copyright 2014-2022 Dimitris Zlatanidis <d.zlatanidis@gmail.com>
# All rights reserved.

# Slpkg is a user-friendly package manager for Slackware installations

# https://gitlab.com/dslackw/slpkg

# Slpkg is free software: you can redistribute it and/or modify
# it under the terms of the GNU General Public License as published by
# the Free Software Foundation, either version 3 of the License, or
# (at your option) any later version.
# This program is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. See the
# GNU General Public License for more details.
# You should have received a copy of the GNU General Public License
# along with this program. If not, see <http://www.gnu.org/licenses/>.


import os

from slpkg.messages import Msg
from slpkg.__metadata__ import MetaData as _meta_

from slpkg.pkg.find import find_package


class PackageHealth:
    """Health check installed packages
    """
    def __init__(self, mode):
        self.mode = mode
        self.meta = _meta_
        self.green = _meta_.color["GREEN"]
        self.red = _meta_.color["RED"]
        self.yellow = _meta_.color["YELLOW"]
        self.endc = _meta_.color["ENDC"]
        self.msg = Msg()
        self.pkg_path = _meta_.pkg_path
        self.installed = []
        self.cn = 0

    def packages(self):
        """Get all installed packages from /var/log/packages/ path
        """
        self.installed = find_package("", self.pkg_path)

    def check(self, line, pkg):
        line = line.replace("\n", "")
        try:
            if (not line.endswith("/") and
                    not line.endswith(".new") and
                    not line.startswith("dev/") and
                    not line.startswith("install/") and
                    "/incoming/" not in line):
                if not os.path.isfile(r"/" + line):
                    self.cn += 1
                    print(f"Not installed: {self.red}/{line}{self.endc} --> {pkg}")
                elif not self.mode:
                    print(line)
        except IOError:
            raise SystemExit("\n")

    def test(self):
        """Get started test each package and read file list
        """
        self.packages()
        self.cf = 0
        for pkg in self.installed:
            if os.path.isfile(f"{self.meta.pkg_path}{pkg}"):
                self.lf = 0
                with open(self.pkg_path + pkg, "r") as fopen:
                    for line in fopen:
                        if "\0" in line:
                            print(f"Null: {line}")
                            break
                        self.cf += 1     # count all files
                        self.lf += 1     # count each package files
                        if self.lf > 19:
                            self.check(line, pkg)
        self.results()

    def results(self):
        """Print results
        """
        print()
        per = int(round((float(self.cf) / (self.cf + self.cn)) * 100))
        if per > 90:
            color = self.green
        elif per < 90 and per > 60:
            color = self.yellow
        elif per < 60:
            color = self.red
        health = f"{color}{str(per)}%{self.endc}"
        self.msg.template(78)
        print(f"| Total files{' ' * 7}Not installed{' ' * 40}Health")
        self.msg.template(78)
        print(f"| {self.cf}{' ' * (18-len(str(self.cf)))}{self.cn}{' ' * (55-len(str(self.cn)))}{health:>4}")
        self.msg.template(78)



import unittest
from slpkg.pkg.installed import GetFromInstalled


class TestPkgInstalled(unittest.TestCase):

    def setUp(self):
        self.pkg_name = 'aaa_base'
        self.pkg_ver = '15.0'
        self.installed = GetFromInstalled('aaa_base')

    def test_pkg_name(self):
        """Testing the installed package name
        """
        self.assertEqual(self.pkg_name, self.installed.name())

    def test_pkg_version(self):
        """Testing the version of installed package"""
        self.assertEqual(self.pkg_ver, self.installed.version())


if __name__ == "__main__":
    unittest.main()



#!/usr/bin/python3
# -*- coding: utf-8 -*-

# repositories.py file is part of slpkg.

# Copyright 2014-2022 Dimitris Zlatanidis <d.zlatanidis@gmail.com>
# All rights reserved.

# Slpkg is a user-friendly package manager for Slackware installations

# https://gitlab.com/dslackw/slpkg

# Slpkg is free software: you can redistribute it and/or modify
# it under the terms of the GNU General Public License as published by
# the Free Software Foundation, either version 3 of the License, or
# (at your option) any later version.
# This program is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. See the
# GNU General Public License for more details.
# You should have received a copy of the GNU General Public License
# along with this program. If not, see <http://www.gnu.org/licenses/>.


import os

from slpkg.utils import Utils
from slpkg.__metadata__ import MetaData as _meta_


class Repo(Utils):
    """Manage repositories configuration files
    """
    def __init__(self):
        self.meta = _meta_
        self.DEFAULT_REPOS_NAMES = self.meta.default_repositories
        self.custom_repo_file = "/etc/slpkg/custom-repositories"
        self.default_repo_file = "/etc/slpkg/default-repositories"
        self.custom_repositories_list = self.read_file(
            self.custom_repo_file)
        self.default_repositories_list = self.read_file(
            self.default_repo_file)
        self.default_repository()

    def add(self, repo, url):
        """Write custom repository name and url in a file
        """
        repo_name = []
        if not url.endswith("/"):
            url += "/"
        for line in self.custom_repositories_list.splitlines():
            line = line.lstrip()
            if line and not line.startswith("#"):
                repo_name.append(line.split()[0])
        if (repo in self.meta.repositories or repo in repo_name or
                repo in self.meta.default_repositories):
            raise SystemExit(f"\nRepository name '{repo}' exist, "
                             f"select different name.\n"
                             f"View all repositories with command 'slpkg "
                             f"repo-list'.\n")
        elif len(repo) > 6:
            raise SystemExit("\nslpkg: Error: Maximum repository name length "
                  "must be six (6) characters\n")
        with open(self.custom_repo_file, "a") as repos:
            new_line = f"  {repo}{' ' * (10 - len(repo))}{url}\n"
            repos.write(new_line)
        print(f"\nRepository '{repo}' successfully added\n")

    def remove(self, repo):
        """Remove custom repository
        """
        rem_repo = False
        with open(self.custom_repo_file, "w") as repos:
            for line in self.custom_repositories_list.splitlines():
                repo_name = line.split()[0]
                if repo_name != repo:
                    repos.write(f"{line}\n")
                else:
                    print(f"\nRepository '{repo}' successfully "
                          "removed\n")
                    rem_repo = True
        if not rem_repo:
            print(f"\nRepository '{repo}' doesn't exist\n")

    def custom_repository(self):
        """Return dictionary with repo name and url (used external)
        """
        custom_dict_repo = {}
        for line in self.custom_repositories_list.splitlines():
            line = line.lstrip()
            if not line.startswith("#"):
                custom_dict_repo[line.split()[0]] = line.split()[1]
        return custom_dict_repo

    def default_repository(self):
        """Return dictionary with default repo name and url
        """
        default_dict_repo = {}
        for line in self.default_repositories_list.splitlines():
            line = line.lstrip()
            if not line.startswith("#"):
                if line.split()[0] in self.DEFAULT_REPOS_NAMES:
                    default_dict_repo[line.split()[0]] = line.split()[1]
                else:
                    raise SystemExit(f"\nslpkg: Error: Repository name "
                                     f"'{line.split()[0]}'"
                                     f" is not default.\n              "
                                     f"Please check file: "
                                     f"/etc/slpkg/default-repositories\n")
        return default_dict_repo

    def slack(self):
        """Official slackware repository
        """
        default = "http://mirrors.slackware.com/slackware/"
        if self.meta.arch.startswith("arm"):
            default = "http://ftp.arm.slackware.com/slackwarearm/"
        if os.path.isfile("/etc/slpkg/slackware-mirrors"):
            mirrors = self.read_file(
                f"{self.meta.conf_path}slackware-mirrors")
            for line in mirrors.splitlines():
                line = line.rstrip()
                if not line.startswith("#") and line:
                    default = line.split()[-1]
        if not default.endswith("/"):
            default += "/"
        return default



#!/usr/bin/python3
# -*- coding: utf-8 -*-

# install.py file is part of slpkg.

# Copyright 2014-2022 Dimitris Zlatanidis <d.zlatanidis@gmail.com>
# All rights reserved.

# Slpkg is a user-friendly package manager for Slackware installations

# https://gitlab.com/dslackw/slpkg

# Slpkg is free software: you can redistribute it and/or modify
# it under the terms of the GNU General Public License as published by
# the Free Software Foundation, either version 3 of the License, or
# (at your option) any later version.
# This program is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. See the
# GNU General Public License for more details.
# You should have received a copy of the GNU General Public License
# along with this program. If not, see <http://www.gnu.org/licenses/>.


import os
from itertools import zip_longest
from pkg_resources import parse_version

from slpkg.utils import Utils
from slpkg.sizes import units
from slpkg.messages import Msg
from slpkg.checksum import check_md5
from slpkg.blacklist import BlackList
from slpkg.downloader import Download
from slpkg.log_deps import write_deps
from slpkg.grep_md5 import pkg_checksum
from slpkg.remove import delete_package
from slpkg.security import pkg_security
from slpkg.splitting import split_package
from slpkg.__metadata__ import MetaData as _meta_

from slpkg.pkg.find import find_package
from slpkg.pkg.manager import PackageManager
from slpkg.pkg.installed import GetFromInstalled

from slpkg.binary.greps import repo_data
from slpkg.binary.repo_init import RepoInit
from slpkg.binary.dependency import Dependencies


class BinaryInstall(BlackList, Utils):
    """Install binaries packages with all dependencies from
    repository
    """
    def __init__(self, packages, repo, flag):
        super().__init__()
        self.packages = packages
        pkg_security(packages)
        self.repo = repo
        self.flag = flag
        self.meta = _meta_
        self.green = _meta_.color["GREEN"]
        self.red = _meta_.color["RED"]
        self.grey = _meta_.color["GREY"]
        self.yellow = _meta_.color['YELLOW']
        self.endc = _meta_.color["ENDC"]
        self.msg = Msg()
        self.version = self.meta.slack_rel
        self.tmp_path = self.meta.slpkg_tmp_packages
        self.init_flags()
        self.dwn, self.dep_dwn = [], []
        self.install, self.dep_install = [], []
        self.comp_sum, self.dep_comp_sum = [], []
        self.uncomp_sum, self.dep_uncomp_sum = [], []
        self.dependencies = []
        self.deps_dict = {}
        self.answer = ""
        self.msg.reading()
        self.PACKAGES_TXT, self.mirror = RepoInit(self.repo).fetch()
        self.data = repo_data(self.PACKAGES_TXT, self.repo, self.flag)
        self.repo_pkg_names = []
        for name in self.data[0]:
            self.repo_pkg_names.append(split_package(name)[0])
        self.blacklist = list(self.get_black())
        self.matching = False

    def init_flags(self):
        """Flags initiliazation
        """
        for fl in self.flag:
            if fl.startswith("--directory-prefix="):
                self.tmp_path = fl.split("=")[1]
                if not self.tmp_path.endswith("/"):
                    self.tmp_path += "/"

    def start(self, is_upgrade):
        """
        Install packages from official Slackware distribution
        """
        self.case_insensitive()
        # fix if packages is for upgrade
        self.is_upgrade = is_upgrade
        mas_sum = dep_sum = sums = 0, 0, 0, 0
        self.msg.done()
        self.dependencies = self.resolving_deps()
        self.update_deps()
        (self.dep_dwn, self.dep_install, self.dep_comp_sum,
            self.dep_uncomp_sum) = self.store(self.dependencies)
        self.clear_masters()
        (self.dwn, self.install, self.comp_sum,
            self.uncomp_sum) = self.store(self.packages)
        if (self.meta.rsl_deps in ["on", "ON"] and
                "--resolve-off" not in self.flag):
            self.msg.done()
        if self.install:
            if self.matching and [""] != self.packages:
                self.msg.matching(self.packages)
            else:
                print("\nThe following packages will be automatically "
                      "installed or upgraded \nwith new version:\n")
            self.top_view()
            self.msg.upg_inst(self.is_upgrade)
            mas_sum = self.views(self.install, self.comp_sum)
            if self.dependencies:
                print("Installing for dependencies:")
                dep_sum = self.views(self.dep_install, self.dep_comp_sum)
            # sums[0] --> total packages
            # sums[1] --> reinstall
            # sums[2] --> upgraded
            # sums[3] --> uninstall
            sums = [sum(s) for s in zip_longest(mas_sum, dep_sum)]
            unit, size = units(self.comp_sum + self.dep_comp_sum,
                               self.uncomp_sum + self.dep_uncomp_sum)
            if self.matching and [""] != self.packages:
                print("\nMatching summary")
                print("=" * 79)
                raise SystemExit(f"Total {sums[0]} matching packages\n")
            print("\nInstalling summary")
            print("=" * 79)
            print(f"{self.grey}Total {sums[0]} {self.msg.pkg(sums[0])}.")
            print(f"{sums[3]} {self.msg.pkg(sums[3])} will be installed, "
                  f"{sums[2]} will be upgraded and "
                  f"{sums[1]} will be reinstalled.")
            print(f"Need to get {size[0]} {unit[0]} of archives.")
            print(f"After this process, {size[1]} {unit[1]} of additional "
                  f"disk space will be used.{self.endc}")
            print()
            self.if_all_installed()
            if self.msg.answer() in ["y", "Y"]:
                for inst, dwn in zip(self.dep_install + self.install,
                                     self.dep_dwn + self.dwn):
                    if (self.meta.not_downgrade == "on" and
                            self.not_downgrade(inst) is True):
                        continue
                    if (not os.path.isfile(self.meta.pkg_path + inst[:-4]) or
                            "--download-only" in self.flag or
                            "--reinstall" in self.flag):
                        Download(self.tmp_path, dwn.split(), self.repo).start()
                    else:
                        self.msg.template(78)
                        self.msg.pkg_found(inst)
                        self.msg.template(78)
                if "--download-only" in self.flag:
                    raise SystemExit()
                self.dep_install = list(self.check_downloaded(
                    self.tmp_path, self.dep_install))
                self.install = list(self.check_downloaded(
                    self.tmp_path, self.install))
                ins, upg = self.install_packages()
                self.msg.reference(ins, upg)
                write_deps(self.deps_dict)
                delete_package(self.tmp_path, self.dep_install + self.install)
        else:
            self.msg.not_found(self.is_upgrade)
            raise SystemExit(1)

    def if_all_installed(self):
        """Check if all packages is already installed
        """
        count_inst = 0
        for inst in (self.dep_install + self.install):
            if (os.path.isfile(self.meta.pkg_path + inst[:-4]) and
                    "--download-only" not in self.flag):
                count_inst += 1
        if (count_inst == len(self.dep_install + self.install) and
                "--reinstall" not in self.flag):
            raise SystemExit()

    def case_insensitive(self):
        """Matching packages distinguish between uppercase and
        lowercase
        """
        if "--case-ins" in self.flag:
            data = list(self.package_name(self.PACKAGES_TXT))
            data_dict = self.case_sensitive(data)
            for pkg in self.packages:
                index = self.packages.index(pkg)
                for key, value in data_dict.items():
                    if key == pkg.lower():
                        self.packages[index] = value

    def update_deps(self):
        """Update dependencies dictionary with all package
        """
        for dep in self.dependencies:
            deps = self.dimensional_list(Dependencies(
                self.repo, self.blacklist).binary(
                    dep, self.flag))
            self.deps_dict[dep] = deps

    def clear_masters(self):
        """Clear master packages if already exist in dependencies
        or if added to install two or more times
        """
        packages = []
        for mas in self.remove_dbs(self.packages):
            if mas not in self.dependencies:
                packages.append(mas)
        self.packages = packages

    def install_packages(self):
        """Install or upgrade packages
        """
        installs, upgraded = [], []
        for inst in (self.dep_install + self.install):
            package = (self.tmp_path + inst).split()
            pkg_ver = f"{split_package(inst)[0]}-{split_package(inst)[1]}"
            self.checksums(inst)
            if GetFromInstalled(split_package(inst)[0]).name():
                print(f"[ {self.yellow}upgrading{self.endc} ] --> {inst}")
                upgraded.append(pkg_ver)
                if "--reinstall" in self.flag:
                    PackageManager(package).upgrade("--reinstall")
                else:
                    PackageManager(package).upgrade("--install-new")
            else:
                print(f"[ {self.green}installing{self.endc} ] --> {inst}")
                installs.append(pkg_ver)
                PackageManager(package).upgrade("--install-new")
        return [installs, upgraded]

    def not_downgrade(self, package):
        """Don't downgrade packages if repository version is lower than
        installed"""
        name = split_package(package)[0]
        rep_ver = split_package(package)[1]
        ins_ver = GetFromInstalled(name).version()
        if not ins_ver:
            ins_ver = "0"
        if parse_version(rep_ver) < parse_version(ins_ver):
            self.msg.template(78)
            print(f"| Package {name} does not downgrade, setting by user")
            self.msg.template(78)
            return True

    def checksums(self, install):
        """Checksums before install
        """
        check_md5(pkg_checksum(install, self.repo), self.tmp_path + install)

    def resolving_deps(self):
        """Return package dependencies
        """
        requires = []
        if (self.meta.rsl_deps in ["on", "ON"] and
                self.flag != "--resolve-off"):
            self.msg.resolving()
        for dep in self.packages:
            dependencies = []
            dependencies = self.dimensional_list(Dependencies(
                self.repo, self.blacklist).binary(dep, self.flag))
            requires += list(self._fix_deps_repos(dependencies))
            self.deps_dict[dep] = self.remove_dbs(requires)
        return self.remove_dbs(requires)

    def _fix_deps_repos(self, dependencies):
        """Fix store deps include in repository
        """
        for dep in dependencies:
            if dep in self.repo_pkg_names:
                yield dep

    def views(self, install, comp_sum):
        """Views packages
        """
        pkg_sum = uni_sum = upg_sum = res_sum = 0

        # fix repositories align
        repo = self.repo + (" " * (6 - (len(self.repo))))
        for pkg, comp in zip(install, comp_sum):
            pkg_sum += 1
            pkg_repo = split_package(pkg[:-4])
            if find_package(pkg[:-4], self.meta.pkg_path):
                if "--reinstall" in self.flag:
                    res_sum += 1
                COLOR = self.meta.color["GREEN"]
            elif pkg_repo[0] == GetFromInstalled(pkg_repo[0]).name():
                COLOR = self.meta.color["YELLOW"]
                upg_sum += 1
            else:
                COLOR = self.meta.color["RED"]
                uni_sum += 1
            ver = GetFromInstalled(pkg_repo[0]).version()
            if ver:
                ver = f"-{ver}"
            print(f"  {COLOR}{pkg_repo[0] + ver}{self.endc}"
                  f"{' ' * (23-len(pkg_repo[0] + ver))} {pkg_repo[1]}"
                  f"{' ' * (18-len(pkg_repo[1]))} {pkg_repo[2]}"
                  f"{' ' * (8-len(pkg_repo[2]))}{pkg_repo[3]}"
                  f"{' ' * (7-len(pkg_repo[3]))}{repo}{comp:>11}{' K'}")
        return [pkg_sum, res_sum, upg_sum, uni_sum]

    def top_view(self):
        """Print packages status bar
        """
        self.msg.template(78)
        print(f"| Package{' ' * 17}New Version{' ' * 8}Arch{' ' * 4}"
              f"Build{' ' * 2}Repos{' ' * 10}Size")
        self.msg.template(78)

    def store(self, packages):
        """Store and return packages for install
        """
        dwn, install, comp_sum, uncomp_sum = ([] for i in range(4))
        # name = data[0]
        # location = data[1]
        # size = data[2]
        # unsize = data[3]
        for pkg in packages:
            for pk, loc, comp, uncomp in zip(self.data[0], self.data[1],
                                             self.data[2], self.data[3]):
                if (pk and pkg == split_package(pk)[0] and
                        pk not in install and
                        split_package(pk)[0] not in self.blacklist):
                    dwn.append(f"{self.mirror}{loc}/{pk}")
                    install.append(pk)
                    comp_sum.append(comp)
                    uncomp_sum.append(uncomp)

        if not install:
            for pkg in packages:
                for pk, loc, comp, uncomp in zip(self.data[0], self.data[1],
                                                 self.data[2], self.data[3]):
                    name = split_package(pk)[0]
                    if (pk and pkg in name and name not in self.blacklist):
                        self.matching = True
                        dwn.append(f"{self.mirror}{loc}/{pk}")
                        install.append(pk)
                        comp_sum.append(comp)
                        uncomp_sum.append(uncomp)
        dwn.reverse()
        install.reverse()
        comp_sum.reverse()
        uncomp_sum.reverse()

        return [dwn, install, comp_sum, uncomp_sum]



#!/usr/bin/python3
# -*- coding: utf-8 -*-

# dialog_box.py file is part of slpkg.

# Copyright 2014-2022 Dimitris Zlatanidis <d.zlatanidis@gmail.com>
# All rights reserved.

# Slpkg is a user-friendly package manager for Slackware installations

# https://gitlab.com/dslackw/slpkg

# Slpkg is free software: you can redistribute it and/or modify
# it under the terms of the GNU General Public License as published by
# the Free Software Foundation, either version 3 of the License, or
# (at your option) any later version.
# This program is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. See the
# GNU General Public License for more details.
# You should have received a copy of the GNU General Public License
# along with this program. If not, see <http://www.gnu.org/licenses/>.


from __future__ import unicode_literals

import os


class DialogUtil:
    """Create dialog checklist
    """
    def __init__(self, *args):
        self.imp_dialog()
        self.data = args[0]
        self.text = args[1]
        self.title = args[2]
        self.backtitle = args[3]
        self.status = args[4]
        self.ununicode = []
        self.tags = []

    def imp_dialog(self):
        try:
            from dialog import Dialog
        except ImportError as er:
            raise SystemExit(er)
        self.d = Dialog(dialog="dialog", autowidgetsize=True)

    def checklist(self):
        """Run dialog checklist
        """
        choice = []
        for item in self.data:
            choice.append((item, "", self.status))
        code, self.tags = self.d.checklist(
            text=self.text, height=20, width=65, list_height=13,
            choices=choice, title=self.title, backtitle=self.backtitle)
        if code == "ok":
            self.unicode_to_string()
            return self.ununicode
        if code in ["cancel", "esc"]:
            self.exit()

    def buildlist(self, enabled):
        """Run dialog buildlist
        """
        choice = []
        for item in self.data:
            choice.append((item, False))
        for item in enabled:
            choice.append((item, True))
        items = [(tag, tag, sta) for (tag, sta) in choice]
        code, self.tags = self.d.buildlist(
            text=self.text, items=items, visit_items=True, item_help=False,
            title=self.title)
        if code == "ok":
            self.unicode_to_string()
            return self.ununicode
        if code in ["cancel", "esc"]:
            self.exit()

    def exit(self):
        """Exit from dialog
        """
        self.clear_screen()
        raise SystemExit()

    def clear_screen(self):
        """Clear screen
        """
        os.system("clear")

    def unicode_to_string(self):
        """Convert unicode in string
        """
        for tag in self.tags:
            self.ununicode.append(str(tag))



#!/usr/bin/python3
# -*- coding: utf-8 -*-

# autobuild.py file is part of slpkg.

# Copyright 2014-2022 Dimitris Zlatanidis <d.zlatanidis@gmail.com>
# All rights reserved.

# Slpkg is a user-friendly package manager for Slackware installations

# https://gitlab.com/dslackw/slpkg

# Slpkg is free software: you can redistribute it and/or modify
# it under the terms of the GNU General Public License as published by
# the Free Software Foundation, either version 3 of the License, or
# (at your option) any later version.
# This program is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. See the
# GNU General Public License for more details.
# You should have received a copy of the GNU General Public License
# along with this program. If not, see <http://www.gnu.org/licenses/>.


import os

from slpkg.pkg.build import BuildPackage

from slpkg.sbo.greps import SBoGrep


class AutoBuild:
    """Autobuild package if sources and script is already
    downloaded
    """
    def __init__(self, script, sources, path):
        self.script = script
        self.sources = sources
        self.prgnam = self.script[:-7]
        self.path = path
        self.sbo_sources = []

    def run(self):
        """Build package and fix ordelist per checksum
        """
        self.files_exist()
        self.info_file()
        sources = self.sources
        if len(sources) > 1 and self.sbo_sources != sources:
            sources = self.sbo_sources
        # If the list does not have the same order use from .info
        # order.
        BuildPackage(self.script, sources, self.path, auto=True).build()
        raise SystemExit()

    def info_file(self):
        """Grab sources from .info file and store filename
        """
        sources = SBoGrep(self.prgnam).source().split()
        for source in sources:
            self.sbo_sources.append(source.split("/")[-1])

    def files_exist(self):
        """Check if SlackBuild archive.tar.gz and sources exist
        """
        if not os.path.isfile(self.path + self.script):
            raise SystemExit("\nslpkg: Error: SlackBuild archive.tar.gz "
                             "not found\n")
        for src in self.sources:
            if not os.path.isfile(self.path + src):
                raise SystemExit(f"\nslpkg: Error: Source file '{src}' "
                                 f"not found\n")



#!/usr/bin/python3
# -*- coding: utf-8 -*-

# mirrors.py file is part of slpkg.

# Copyright 2014-2022 Dimitris Zlatanidis <d.zlatanidis@gmail.com>
# All rights reserved.

# Slpkg is a user-friendly package manager for Slackware installations

# https://gitlab.com/dslackw/slpkg

# Slpkg is free software: you can redistribute it and/or modify
# it under the terms of the GNU General Public License as published by
# the Free Software Foundation, either version 3 of the License, or
# (at your option) any later version.
# This program is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. See the
# GNU General Public License for more details.
# You should have received a copy of the GNU General Public License
# along with this program. If not, see <http://www.gnu.org/licenses/>.


from slpkg.repositories import Repo
from slpkg.__metadata__ import MetaData as _meta_

from slpkg.slack.slack_version import slack_ver


def mirrors(name, location):
    """Select Slackware official mirror packages
       based architecture and version.
    """
    rel = _meta_.slack_rel
    ver = slack_ver()
    repo = Repo()
    slack = repo.slack()
    if _meta_.arch == "x86_64":
        if rel == "stable":
            http = f"{slack}slackware64-{ver}/{location}{name}"
        else:
            http = f"{slack}slackware64-{rel}/{location}{name}"
    elif _meta_.arch.startswith("arm"):
        if rel == "stable":
            http = f"{slack}slackwarearm-{ver}/{location}{name}"
        else:
            http = f"{slack}slackwarearm-{rel}/{location}{name}"
    else:
        if rel == "stable":
            http = f"{slack}slackware-{ver}/{location}{name}"
        else:
            http = f"{slack}slackware-{rel}/{location}{name}"
    return http



#!/usr/bin/python3
# -*- coding: utf-8 -*-

# tracking.py file is part of slpkg.

# Copyright 2014-2022 Dimitris Zlatanidis <d.zlatanidis@gmail.com>
# All rights reserved.

# Slpkg is a user-friendly package manager for Slackware installations

# https://gitlab.com/dslackw/slpkg

# Slpkg is free software: you can redistribute it and/or modify
# it under the terms of the GNU General Public License as published by
# the Free Software Foundation, either version 3 of the License, or
# (at your option) any later version.
# This program is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. See the
# GNU General Public License for more details.
# You should have received a copy of the GNU General Public License
# along with this program. If not, see <http://www.gnu.org/licenses/>.


from slpkg.utils import Utils
from slpkg.graph import Graph
from slpkg.messages import Msg
from slpkg.blacklist import BlackList
from slpkg.__metadata__ import MetaData as _meta_

from slpkg.pkg.find import find_package

from slpkg.sbo.greps import SBoGrep
from slpkg.sbo.dependency import Requires
from slpkg.sbo.search import sbo_search_pkg

from slpkg.binary.search import search_pkg
from slpkg.binary.dependency import Dependencies


class TrackingDeps(BlackList, Utils):
    """View tree of dependencies and also
    highlight packages with color green
    if already installed and color red
    if not installed.
    """
    def __init__(self, name, repo, flag):
        super().__init__()
        self.name = name
        self.repo = repo
        self.flag = flag
        self.meta = _meta_
        self.msg = Msg()
        self.green = self.meta.color["GREEN"]
        self.yellow = self.meta.color["YELLOW"]
        self.cyan = self.meta.color["CYAN"]
        self.red = self.meta.color["RED"]
        self.endc = self.meta.color["ENDC"]
        self.requires = []
        self.dependencies = []
        self.dependencies_list = []
        self.deps_dict = {}
        self.init_flags()

    def init_flags(self):
        """Flags initialization
        """
        for i in range(0, len(self.flag)):
            if self.flag[i].startswith("--graph="):
                self.image = self.flag[i].split("=")[1]
                self.flag[i] = "--graph="

    def run(self):
        """Run tracking dependencies
        """
        self.msg.resolving()
        self.repositories()
        if self.find_pkg:
            self.dependencies_list.reverse()
            self.requires = self.dimensional_list(self.dependencies_list)
            self.dependencies = self.remove_dbs(self.requires)
            if self.dependencies == []:
                self.dependencies = ["No dependencies"]
            if "--graph=" in self.flag:
                self.deps_tree()
            self.msg.done()
            pkg_len = len(self.name) + 24
            print()    # new line at start
            self.msg.template(pkg_len)
            print(f"| Package {self.cyan}{self.name}{self.endc} "
                  f"dependencies :")
            self.msg.template(pkg_len)
            print("\\")
            print(f" +---{self.yellow}[ Tree of dependencies ]{self.endc}")
            index = 0
            for pkg in self.dependencies:
                if "--check-deps" in self.flag:
                    used = self.check_used(pkg)
                    self.deps_used(pkg, used)
                    used = (f"is dependence on --> "
                            f"{self.cyan}{', '.join(used)}{self.endc}")
                else:
                    used = ""
                index += 1
                installed = ""
                if find_package(f"{pkg}-", self.meta.pkg_path):
                    if self.meta.use_colors in ["off", "OFF"]:
                        installed = "* "
                    print(" |")
                    print(f" +--{index}: {self.green}{pkg}{self.endc} "
                          f"{installed}{used}")
                else:
                    print(" |")
                    print(f" +--{index}: {self.red}{pkg}{self.endc} "
                          f"{installed}")
            if self.meta.use_colors in ["off", "OFF"]:
                print("\n * = Installed\n")
            else:
                print()    # new line at end
            if "--graph=" in self.flag:
                self.graph()
        else:
            self.msg.done()
            raise SystemExit("\nNo package was found to match\n")

    def repositories(self):
        """Get dependencies by repositories
        """
        if self.repo == "sbo":
            self.sbo_case_insensitive()
            self.find_pkg = sbo_search_pkg(self.name)
            if self.find_pkg:
                self.dependencies_list = Requires(self.flag).sbo(self.name)
        else:
            PACKAGES_TXT = self.read_file(
                f"{self.meta.lib_path}{self.repo}_repo/PACKAGES.TXT")
            self.names = list(self.package_name(PACKAGES_TXT))
            self.bin_case_insensitive()
            self.find_pkg = search_pkg(self.name, self.repo)
            if self.find_pkg:
                self.black = list(self.get_black())
                self.dependencies_list = Dependencies(
                    self.repo, self.black).binary(self.name, self.flag)

    def sbo_case_insensitive(self):
        """Matching packages distinguish between uppercase and
        lowercase for sbo repository
        """
        if "--case-ins" in self.flag:
            data = SBoGrep(name="").names()
            data_dict = self.case_sensitive(data)
            for key, value in data_dict.items():
                if key == self.name.lower():
                    self.name = value

    def bin_case_insensitive(self):
        """Matching packages distinguish between uppercase and
        lowercase
        """
        if "--case-ins" in self.flag:
            data_dict = self.case_sensitive(self.names)
            for key, value in data_dict.items():
                if key == self.name.lower():
                    self.name = value

    def graph(self):
        """Drawing image dependencies map
        """
        Graph(self.image).dependencies(self.deps_dict)

    def check_used(self, pkg):
        """Check if dependencies used
        """
        used = []
        dep_path = f"{self.meta.log_path}dep/"
        logs = find_package("", dep_path)
        for log in logs:
            deps = self.read_file(f"{dep_path}{log}")
            for dep in deps.splitlines():
                if pkg == dep:
                    used.append(log)
        return used

    def deps_tree(self):
        """Package dependencies image map file
        """
        dependencies = self.dependencies + [self.name]
        if self.repo == "sbo":
            for dep in dependencies:
                deps = Requires(flag="").sbo(dep)
                if dep not in self.deps_dict.values():
                    self.deps_dict[dep] = self.dimensional_list(deps)
        else:
            for dep in dependencies:
                deps = Dependencies(self.repo, self.black).binary(dep, flag="")
                if dep not in self.deps_dict.values():
                    self.deps_dict[dep] = self.dimensional_list(deps)

    def deps_used(self, pkg, used):
        """Create dependencies dictionary
        """
        if find_package(f"{pkg}-", self.meta.pkg_path):
            if pkg not in self.deps_dict.values():
                self.deps_dict[pkg] = used
            else:
                self.deps_dict[pkg] += used



#!/usr/bin/python3
# -*- coding: utf-8 -*-

# repo_init.py file is part of slpkg.

# Copyright 2014-2022 Dimitris Zlatanidis <d.zlatanidis@gmail.com>
# All rights reserved.

# Slpkg is a user-friendly package manager for Slackware installations

# https://gitlab.com/dslackw/slpkg

# Slpkg is free software: you can redistribute it and/or modify
# it under the terms of the GNU General Public License as published by
# the Free Software Foundation, either version 3 of the License, or
# (at your option) any later version.
# This program is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. See the
# GNU General Public License for more details.
# You should have received a copy of the GNU General Public License
# along with this program. If not, see <http://www.gnu.org/licenses/>.


from slpkg.utils import Utils
from slpkg.repositories import Repo
from slpkg.__metadata__ import MetaData as _meta_

from slpkg.slack.mirrors import mirrors
from slpkg.slack.slack_version import slack_ver


class RepoInit(Utils):
    """Return PACKAGES.TXT and mirror by repository
    """
    def __init__(self, repo):
        self.repo = repo
        self.meta = _meta_
        self.def_repo_dict = Repo().default_repository()
        self.mirror = ""

    def fetch(self):
        if self.repo in self.meta.default_repositories:
            getattr(self, f"_init_{self.repo}")()
        else:
            self._init_custom()
        self.lib = self.meta.lib_path + f"{self.repo}_repo/PACKAGES.TXT"
        PACKAGES_TXT = self.read_file(self.lib)
        return PACKAGES_TXT, self.mirror

    def _init_custom(self):
        repo = Repo()
        self.mirror = repo.custom_repository()[self.repo]

    def _init_slack(self):
        self.mirror = mirrors(name="", location="")

    def _init_rlw(self):
        self.mirror = f"{self.def_repo_dict}{slack_ver()}/"

    def _init_alien(self):
        ver = slack_ver()
        arch = "x86"
        if self.meta.arch == "x86_64":
            arch = "x86_64"
        if self.meta.slack_rel == "current":
            ver = self.meta.slack_rel
        self.mirror = f"{self.def_repo_dict['alien']}{ver}/{arch}/"

    def _init_slacky(self):
        arch = ""
        if self.meta.arch == "x86_64":
            arch = "64"
        self.mirror = f"{self.def_repo_dict}slackware{arch}-{slack_ver()}/"

    def _init_conrad(self):
        self.mirror = self.def_repo_dict["conrad"]

    def _init_slonly(self):
        ver = slack_ver()
        arch = f"{ver}-x86"
        if self.meta.arch == "x86_64":
            arch = f"{ver}-x86_64"
        if self.meta.slack_rel == "current":
            if self.meta.arch == "x86_64":
                arch = f"{self.meta.slack_rel}-x86_64"
            else:
                arch = f"{self.meat.slack_rel}-x86"
        self.mirror = f"{self.def_repo_dict['slonly']}{arch}/"

    def _init_ktown(self):
        self.mirror = self.def_repo_dict["ktown"]

    def _init_multi(self):
        ver = slack_ver()
        if self.meta.slack_rel == "current":
            ver = self.meta.slack_rel
        self.mirror = self.def_repo_dict["multi"] + ver + "/"

    def _init_slacke(self):
        arch = ""
        if self.meta.arch == "x86_64":
            arch = "64"
        self.mirror = (f"{self.def_repo_dict['slacke']}"
                       f"slacke{self.meta.slacke_sub_repo[1:-1]}/slackware"
                       f"{arch}-{slack_ver()}/")

    def _init_salix(self):
        arch = "i486"
        if self.meta.arch == "x86_64":
            arch = "x86_64"
        self.mirror = f"{self.def_repo_dict['salix']}{arch}/{slack_ver()}/"

    def _init_slackl(self):
        arch = "i486"
        if self.meta.arch == "x86_64":
            arch = "x86_64"
        self.mirror = f"{self.def_repo_dict['slackl']}{arch}/current/"

    def _init_rested(self):
        self.mirror = self.def_repo_dict["rested"]

    def _init_msb(self):
        arch = "x86"
        if self.meta.arch == "x86_64":
            arch = "x86_64"
            self.mirror = (f"{self.def_repo_dict['msb']}{slack_ver()}/"
                           f"{self.meta.msb_sub_repo[1:-1]}/{arch}/")

    def _init_csb(self):
        arch = "x86"
        if self.meta.arch == "x86_64":
            arch = "x86_64"
        self.mirror = f"{self.def_repo_dict['csb']}{slack_ver()}/{arch}"

    def _init_connos(self):
        arch = ""
        if self.meta.arch == "x86_64":
            arch = "64"
            self.mirror = (f"{self.def_repo_dict['connos']}slack-n-free"
                           f"{arch}-{slack_ver()}/")

    def _init_mles(self):
        arch = "32"
        if self.meta.arch == "x86_64":
            arch = "64"
            self.mirror = (f"{self.def_repo_dict['mles']}"
                           f"{self.meta.mles_sub_repo[1:-1]}-"
                           f"{slack_ver()}-{arch}bit/")



#!/usr/bin/python3
# -*- coding: utf-8 -*-

# slackbuild.py file is part of slpkg.

# Copyright 2014-2022 Dimitris Zlatanidis <d.zlatanidis@gmail.com>
# All rights reserved.

# Slpkg is a user-friendly package manager for Slackware installations

# https://gitlab.com/dslackw/slpkg

# Slpkg is free software: you can redistribute it and/or modify
# it under the terms of the GNU General Public License as published by
# the Free Software Foundation, either version 3 of the License, or
# (at your option) any later version.
# This program is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. See the
# GNU General Public License for more details.
# You should have received a copy of the GNU General Public License
# along with this program. If not, see <http://www.gnu.org/licenses/>.


import os
from pkg_resources import parse_version

from slpkg.utils import Utils
from slpkg.messages import Msg
from slpkg.log_deps import write_deps
from slpkg.blacklist import BlackList
from slpkg.downloader import Download
from slpkg.remove import delete_folder
from slpkg.security import pkg_security
from slpkg.__metadata__ import MetaData as _meta_

from slpkg.pkg.find import find_package
from slpkg.pkg.build import BuildPackage
from slpkg.pkg.manager import PackageManager
from slpkg.pkg.installed import GetFromInstalled

from slpkg.sbo.greps import SBoGrep
from slpkg.sbo.sbo_arch import SBoArch
from slpkg.sbo.compressed import SBoLink
from slpkg.sbo.dependency import Requires
from slpkg.sbo.search import sbo_search_pkg
from slpkg.sbo.slack_find import slack_package


class SBoInstall(BlackList, Utils):
    """Build and install SBo packages with all dependencies
    """
    def __init__(self, slackbuilds, flag):
        super().__init__()
        self.slackbuilds = slackbuilds
        pkg_security(self.slackbuilds)
        self.flag = flag
        self.meta = _meta_
        self.green = _meta_.color["GREEN"]
        self.red = _meta_.color["RED"]
        self.yellow = _meta_.color["YELLOW"]
        self.grey = _meta_.color["GREY"]
        self.endc = _meta_.color["ENDC"]
        self.msg = Msg()
        self.arch = SBoArch().get()
        self.build_folder = self.meta.build_path
        self._SOURCES = self.meta.SBo_SOURCES
        self.init_flags()
        self.unst = ["UNSUPPORTED", "UNTESTED"]
        self.master_packages = []
        self.deps = []
        self.dependencies = []
        self.package_not_found = []
        self.package_found = []
        self.deps_dict = {}
        self.answer = ""
        self.match = False
        self.count_ins = 0
        self.count_upg = 0
        self.count_uni = 0
        self.msg.reading()
        self.data = SBoGrep(name="").names()
        self.blacklist = list(self.get_black())

    def init_flags(self):
        """Flags initialization
        """
        for fl in self.flag:
            if fl.startswith("--directory-prefix="):
                self.build_folder = fl.split("=")[1]
                if not self.build_folder.endswith("/"):
                    self.build_folder += "/"

    def start(self, is_upgrade):
        """Start view, build and install SBo packages
        """
        tagc = ""
        self.is_upgrade = is_upgrade
        self.case_insensitive()
        for _sbo in self.slackbuilds:
            if _sbo in self.data and _sbo not in self.blacklist:
                sbo_deps = Requires(self.flag).sbo(_sbo)
                self.deps += sbo_deps
                self.deps_dict[_sbo] = self.one_for_all(sbo_deps)
                self.package_found.append(_sbo)
            else:
                self.package_not_found.append(_sbo)
        self.update_deps()

        if not self.package_found:
            self.match = True
            self.matching()

        self.master_packages, mas_src = self.sbo_version_source(
            self.package_found)
        self.msg.done()
        if (self.meta.rsl_deps in ["on", "ON"] and
                self.flag != "--resolve-off" and not self.match):
            self.msg.resolving()
        self.dependencies, dep_src = self.sbo_version_source(
            self.one_for_all(self.deps))
        if (self.meta.rsl_deps in ["on", "ON"] and
                self.flag != "--resolve-off" and not self.match):
            self.msg.done()
        self.clear_masters()

        if self.package_found:
            if self.match and [""] != self.slackbuilds:
                self.msg.matching(self.slackbuilds)
            else:
                print("\nThe following packages will be automatically "
                      "installed or upgraded \nwith new version:\n")
            self.top_view()
            self.msg.upg_inst(self.is_upgrade)

            # view master packages
            for sbo, arch in zip(self.master_packages, mas_src):
                tagc = self.tag(sbo)
                name = "-".join(sbo.split("-")[:-1])
                self.view_packages(tagc, name, sbo.split("-")[-1],
                                   self.select_arch(arch))
            self.view_installing_for_deps()

            # view dependencies
            for dep, arch in zip(self.dependencies, dep_src):
                tagc = self.tag(dep)
                name = "-".join(dep.split("-")[:-1])
                self.view_packages(tagc, name, dep.split("-")[-1],
                                   self.select_arch(arch))

            count_total = sum([self.count_ins, self.count_upg,
                               self.count_uni])
            if self.match and [""] != self.slackbuilds:
                print("\nMatching summary")
                print("=" * 79)
                raise SystemExit(f"Total {count_total} matching packages\n")
            print("\nInstalling summary")
            print("=" * 79)
            print(f"{self.grey}Total {count_total} "
                  f"{self.msg.pkg(count_total)}.")
            print(f"{self.count_uni} {self.msg.pkg(self.count_uni)} "
                  f"will be installed, "
                  f"{self.count_ins} already installed and "
                  f"{self.count_upg} {self.msg.pkg(self.count_upg)}")
            print(f"will be upgraded.{self.endc}\n")
            self.continue_to_install()
        else:
            self.msg.not_found(self.is_upgrade)
            raise SystemExit()

    def case_insensitive(self):
        """Matching packages distinguish between uppercase and
        lowercase
        """
        if "--case-ins" in self.flag:
            data_dict = self.case_sensitive(self.data)
            for name in self.slackbuilds:
                index = self.slackbuilds.index(name)
                for key, value in data_dict.items():
                    if key == name.lower():
                        self.slackbuilds[index] = value

    def update_deps(self):
        """Update dependencies dictionary with all package
        """
        onelist, dependencies = [], []
        onelist = self.dimensional_list(self.deps)
        dependencies = self.remove_dbs(onelist)
        for dep in dependencies:
            deps = Requires(self.flag).sbo(dep)
            self.deps_dict[dep] = self.one_for_all(deps)

    def continue_to_install(self):
        """Continue to install ?
        """
        if (self.count_uni > 0 or self.count_upg > 0 or
                "--download-only" in self.flag or "--rebuild" in self.flag):
            if self.master_packages and self.msg.answer() in ["y", "Y"]:
                installs, upgraded = self.build_install()
                if "--download-only" in self.flag:
                    raise SystemExit()
                self.msg.reference(installs, upgraded)
                write_deps(self.deps_dict)
                delete_folder(self.build_folder)

    def view_installing_for_deps(self):
        """View installing message for dependencies
        """
        if not self.match and self.dependencies:
            print("Installing for dependencies:")

    def clear_masters(self):
        """Clear master slackbuilds if already exist in dependencies
        or if added to install two or more times
        """
        self.master_packages = self.remove_dbs(self.master_packages)
        for mas in self.master_packages:
            if mas in self.dependencies:
                self.master_packages.remove(mas)

    def matching(self):
        """Return found matching SBo packages
        """
        for sbo in self.package_not_found:
            for pkg in self.data:
                if sbo in pkg and pkg not in self.blacklist:
                    self.package_found.append(pkg)

    def sbo_version_source(self, slackbuilds):
        """Create sbo name with version
        """
        sbo_versions, sources = [], []
        for sbo in slackbuilds:
            sbo_ver = f"{sbo}-{SBoGrep(sbo).version()}"
            sbo_versions.append(sbo_ver)
            sources.append(SBoGrep(sbo).source())
        return [sbo_versions, sources]

    def one_for_all(self, deps):
        """Because there are dependencies that depend on other
        dependencies are created lists into other lists.
        Thus creating this loop create one-dimensional list and
        remove double packages from dependencies.
        """
        requires, dependencies = [], []
        deps.reverse()
        # Inverting the list brings the
        # dependencies in order to be installed.
        requires = self.dimensional_list(deps)
        dependencies = self.remove_dbs(requires)
        return dependencies

    def top_view(self):
        """View top template
        """
        self.msg.template(78)
        print(f"| Packages{' ' * 16}"
              f"New version{' ' * 8}"
              f"Arch{' ' * 4}"
              f"Build{' ' * 2}"
              f"Repos{' ' * 10}"
              f"Size")
        self.msg.template(78)

    def view_packages(self, *args):
        """:View slackbuild packages with version and arch
        args[0] package color
        args[1] package
        args[2] version
        args[3] arch
        """
        ver = GetFromInstalled(args[1]).version()
        if ver:
            ver = f"-{ver}"
        print(f" {args[0]}{args[1] + ver} {self.endc}"
              f"{' ' * (24-len(args[1] + ver))}{args[2]}"
              f"{' ' * (18-len(args[2]))} {args[3]}"
              f"{' ' * (15-len(args[3]))}{''}"
              f"{''}SBo{''}{'':>11}{''}")

    def tag(self, sbo):
        """Tag with color green if package already installed,
        color yellow for packages to upgrade and color red
        if not installed.
        """
        # split sbo name with version and get name
        sbo_name = "-".join(sbo.split("-")[:-1])
        find = GetFromInstalled(sbo_name).name()
        if find_package(sbo, self.meta.pkg_path):
            paint = self.meta.color["GREEN"]
            self.count_ins += 1
            if "--rebuild" in self.flag:
                self.count_upg += 1
        elif sbo_name == find:
            paint = self.meta.color["YELLOW"]
            self.count_upg += 1
        else:
            paint = self.meta.color["RED"]
            self.count_uni += 1
        return paint

    def select_arch(self, src):
        """Looks if sources unsupported or untested
        from arch else select arch.
        """
        arch = self.arch
        for item in self.unst:
            if item in src:
                arch = item
        return arch

    def filenames(self, sources):
        """Return filenames from sources links
        """
        for src in sources:
            yield src.split("/")[-1]

    def build_install(self):
        """Build and install packages if not already installed
        """
        slackbuilds = self.dependencies + self.master_packages
        installs, upgraded, = [], []
        if not os.path.exists(self.build_folder):
            os.makedirs(self.build_folder)
        if not os.path.exists(self._SOURCES):
            os.makedirs(self._SOURCES)
        os.chdir(self.build_folder)
        for prgnam in slackbuilds:
            if (self.meta.not_downgrade == "on" and
                    self.not_downgrade(prgnam) is True):
                continue
            pkg = "-".join(prgnam.split("-")[:-1])
            installed = "".join(find_package(prgnam, self.meta.pkg_path))
            src_link = SBoGrep(pkg).source().split()
            if (installed and "--download-only" not in self.flag and
                    "--rebuild" not in self.flag):
                self.msg.template(78)
                self.msg.pkg_found(prgnam)
                self.msg.template(78)
            elif self.unst[0] in src_link or self.unst[1] in src_link:
                self.msg.template(78)
                print(f"| Package {prgnam} {self.red}"
                      f"{''.join(src_link)}{self.endc}")
                self.msg.template(78)
            else:
                sbo_url = sbo_search_pkg(pkg)
                sbo_link = SBoLink(sbo_url).tar_gz()
                script = sbo_link.split("/")[-1]
                if self.meta.sbosrcarch in ["on", "ON"]:
                    src_link = list(self.sbosrcarsh(prgnam, sbo_link,
                                                    src_link))
                Download(self.build_folder, sbo_link.split(),
                         repo="sbo").start()
                Download(self._SOURCES, src_link, repo="sbo").start()
                if "--download-only" in self.flag:
                    continue
                sources = list(self.filenames(src_link))
                BuildPackage(script, sources, self.build_folder,
                             auto=False).build()
                binary = slack_package(prgnam)
                if os.path.isfile("".join(binary)):
                    if GetFromInstalled(pkg).name() == pkg:
                        print(f"[ {self.yellow}Upgrading{self.endc} ] --> "
                              f"{prgnam}")
                        upgraded.append(prgnam)
                    else:
                        print(f"[ {self.green}Installing{self.endc} ] --> "
                              f"{prgnam}")
                        installs.append(prgnam)
                    if ("--rebuild" in self.flag and
                            GetFromInstalled(pkg).name() == pkg):
                        PackageManager(binary).upgrade(flag="--reinstall")
                    else:
                        PackageManager(binary).upgrade(flag="--install-new")
        return installs, upgraded

    def not_downgrade(self, prgnam):
        """Don't downgrade packages if sbo version is lower than
        installed"""
        name = "-".join(prgnam.split("-")[:-1])
        sbo_ver = prgnam.split("-")[-1]
        ins_ver = GetFromInstalled(name).version()
        if not ins_ver:
            ins_ver = "0"
        if parse_version(sbo_ver) < parse_version(ins_ver):
            self.msg.template(78)
            print(f"| Package {name} does not downgrade, "
                  f"setting by user")
            self.msg.template(78)
            return True

    def sbosrcarsh(self, prgnam, sbo_link, src_link):
        """Alternative repository for sbo sources"""
        name = "-".join(prgnam.split("-")[:-1])
        category = f"{sbo_link.split('/')[-2]}/{name}/"
        for link in src_link:
            source = link.split("/")[-1]
            yield f"{self.meta.sbosrcarch_link}{category}{source}"



import unittest
from slpkg.binary.search import search_pkg
from slpkg.sbo.search import sbo_search_pkg


class TestFindPkg(unittest.TestCase):

    def test_search(self):
        """Testing found the name from binaries repos
        """
        name = "vlc"
        repo = "alien"
        test = search_pkg(name, repo)
        self.assertEqual(name, test)

    def test_sbo_search(self):
        """Testing found the name from binaries repos
        """
        name = "slpkg"
        test = sbo_search_pkg(name).split("/")[-2]
        self.assertEqual(name, test)


if __name__ == "__main__":
    unittest.main()



import unittest
from slpkg.splitting import split_package
from slpkg.pkg.find import searching


class TestSplitting(unittest.TestCase):

    def test_split_pkg(self):
        path = '/var/log/packages/'
        pkg_1 = ''.join(list(searching('slpkg', path)))
        pkg_2 = ''.join(list(searching('akonadi-mime', path)))
        pkg_3 = ''.join(list(searching('autoconf-archive', path)))
        pkg_4 = ''.join(list(searching('bind', path)))

        self.assertListEqual(['slpkg', '3.9.6', 'x86_64', '1'],
                             split_package(pkg_1))
        self.assertListEqual(['akonadi-mime', '21.12.1', 'x86_64', '1'],
                             split_package(pkg_2))
        self.assertListEqual(['autoconf-archive', '2021.02.19', 'noarch', '1'],
                             split_package(pkg_3))
        self.assertListEqual(['bind', '9.16.27', 'x86_64', '1'],
                             split_package(pkg_4))


if __name__ == "__main__":
    unittest.main()



#!/usr/bin/python3
# -*- coding: utf-8 -*-

# network.py file is part of slpkg.

# Copyright 2014-2022 Dimitris Zlatanidis <d.zlatanidis@gmail.com>
# All rights reserved.

# Slpkg is a user-friendly package manager for Slackware installations

# https://gitlab.com/dslackw/slpkg

# Slpkg is free software: you can redistribute it and/or modify
# it under the terms of the GNU General Public License as published by
# the Free Software Foundation, either version 3 of the License, or
# (at your option) any later version.
# This program is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. See the
# GNU General Public License for more details.
# You should have received a copy of the GNU General Public License
# along with this program. If not, see <http://www.gnu.org/licenses/>.


import os
import pydoc

from slpkg.utils import Utils
from slpkg.messages import Msg
from slpkg.blacklist import BlackList
from slpkg.downloader import Download
from slpkg.remove import delete_folder
from slpkg.dialog_box import DialogUtil
from slpkg.security import pkg_security
from slpkg.__metadata__ import MetaData as _meta_

from slpkg.pkg.find import find_package
from slpkg.pkg.build import BuildPackage
from slpkg.pkg.manager import PackageManager

from slpkg.sbo.read import ReadSBo
from slpkg.sbo.greps import SBoGrep
from slpkg.sbo.sbo_arch import SBoArch
from slpkg.sbo.compressed import SBoLink
from slpkg.sbo.search import sbo_search_pkg
from slpkg.sbo.slack_find import slack_package

from slpkg.slack.slack_version import slack_ver


class SBoNetwork(BlackList, Utils):
    """View SBo site in terminal and also read, build or
    install packages
    """
    def __init__(self, name, flag):
        super().__init__()
        self.name = name
        self.flag = flag
        self.meta = _meta_
        self.msg = Msg()
        self.arch = SBoArch().get()
        self.comp_tar = ".tar.gz"
        self.choice = ""
        self.FAULT = ""
        self.green = self.meta.color["GREEN"]
        self.red = self.meta.color["RED"]
        self.yellow = self.meta.color["YELLOW"]
        self.cyan = self.meta.color["CYAN"]
        self.grey = self.meta.color["GREY"]
        self.endc = self.meta.color["ENDC"]
        self.build_folder = self.meta.build_path
        self._SOURCES = self.meta.SBo_SOURCES
        self.msg.reading()
        self.data = SBoGrep(name="").names()
        self.case_insensitive()
        if "--checklist" in self.flag:
            self.with_checklist()
        grep = SBoGrep(self.name)
        self.sbo_files = grep.files()
        self.sbo_url = sbo_search_pkg(self.name)
        if self.sbo_url:
            self.sbo_desc = grep.description()[len(self.name) + 2:-1]
            self.source_dwn = grep.source().split()
            self.sbo_req = grep.requires()
            self.sbo_dwn = SBoLink(self.sbo_url).tar_gz()
            self.sbo_version = grep.version()
            self.dwn_srcs = self.sbo_dwn.split() + self.source_dwn
        if "--checklist" not in self.flag or not self.sbo_url and self.name:
            self.msg.done()

    def view(self):
        """View SlackBuild package, read or install them
        from slackbuilds.org
        """
        if self.sbo_url and self.name not in self.get_black():
            self.prgnam = f"{self.name}-{self.sbo_version}"
            self.view_sbo()
            while True:
                self.read_choice()
                choice = {
                    "r": self.choice_README,
                    "R": self.choice_README,
                    "s": self.choice_SlackBuild,
                    "S": self.choice_SlackBuild,
                    "f": self.choice_info,
                    "F": self.choice_info,
                    "o": self.choice_doinst,
                    "O": self.choice_doinst,
                    "d": self.choice_download,
                    "D": self.choice_download,
                    "download": self.choice_download,
                    "b": self.choice_build,
                    "B": self.choice_build,
                    "build": self.choice_build,
                    "i": self.choice_install,
                    "I": self.choice_install,
                    "install": self.choice_install,
                    "c": self.choice_clear_screen,
                    "C": self.choice_clear_screen,
                    "clear": self.choice_clear_screen,
                    "q": self.choice_quit,
                    "quit": self.choice_quit,
                    "Q": self.choice_quit
                }
                try:
                    choice[self.choice]()
                except KeyError:
                    pass
        else:
            self.msg.pkg_not_found("\n", self.name, "Can't view", "\n")
            raise SystemExit(1)

    def case_insensitive(self):
        """Matching packages distinguish between uppercase and
        lowercase
        """
        if "--case-ins" in self.flag:
            data_dict = self.case_sensitive(self.data)
            for key, value in data_dict.items():
                if key == self.name.lower():
                    self.name = value

    def read_choice(self):
        """Return choice
        """
        commands = {
            "r": "README",
            "R": "README",
            "s": f"{self.name}.SlackBuild",
            "S": f"{self.name}.SlackBuild",
            "f": f"{self.name}.info",
            "F": f"{self.name}.info",
            "o": "doinst.sh",
            "O": "doinst.sh",
            "d": "download",
            "D": "download",
            "download": "download",
            "b": "build",
            "B": "build",
            "build": "build",
            "i": "install",
            "I": "install",
            "install": "install",
            "c": "clear",
            "C": "clear",
            "clear": "clear",
            "q": "quit",
            "quit": "quit",
            "Q": "quit"
        }
        try:
            message = "  Choose an option > "
            self.choice = input(f"{self.grey}{message}{self.endc}")
        except EOFError:
            raise SystemExit('\n')
        try:
            print("{0}\x1b[1A{1}{2}{3}\n".format(
                " " * len(message), self.cyan, commands[self.choice],
                self.endc), end="")
            print(end="", flush=True)
        except KeyError:
            pass

    def choice_README(self):
        """View README file
        """
        README = ReadSBo(self.sbo_url).readme("README")
        fill = self.fill_pager(README)
        self.pager(README + fill)

    def choice_SlackBuild(self):
        """View .SlackBuild file
        """
        SlackBuild = ReadSBo(self.sbo_url).slackbuild(self.name, ".SlackBuild")
        fill = self.fill_pager(SlackBuild)
        self.pager(SlackBuild + fill)

    def choice_info(self):
        """View .info file
        """
        info = ReadSBo(self.sbo_url).info(self.name, ".info")
        fill = self.fill_pager(info)
        self.pager(info + fill)

    def choice_doinst(self):
        """View doinst.sh file
        """
        if "doinst.sh" in self.sbo_files.split():
            doinst_sh = ReadSBo(self.sbo_url).doinst("doinst.sh")
            fill = self.fill_pager(doinst_sh)
            self.pager(doinst_sh + fill)

    def choice_download(self):
        """Download script.tar.gz and sources
        """
        Download(path="", url=self.dwn_srcs, repo="sbo").start()
        raise SystemExit()

    def choice_build(self):
        """Build package
        """
        self.build()
        delete_folder(self.build_folder)
        raise SystemExit()

    def choice_install(self):
        """Download, build and install package
        """
        pkg_security([self.name])
        if not find_package(self.prgnam, self.meta.pkg_path):
            self.build()
            self.install()
            delete_folder(self.build_folder)
            raise SystemExit()
        else:
            self.msg.template(78)
            self.msg.pkg_found(self.prgnam)
            self.msg.template(78)
            raise SystemExit()

    def choice_clear_screen(self):
        """Clear screen
        """
        os.system("clear")
        self.view()

    def choice_quit(self):
        """Quit from choices
        """
        raise SystemExit()

    def view_sbo(self):
        """View slackbuild.org
        """
        sbo_url = self.sbo_url.replace("/slackbuilds/", "/repository/")
        br1, br2, fix_sp = "", "", " "
        if self.meta.use_colors in ["off", "OFF"]:
            br1 = "("
            br2 = ")"
            fix_sp = ""
        print()   # new line at start
        self.msg.template(78)
        print(f"| {' ' * 28}{self.grey}SlackBuilds Repository{self.endc}")
        self.msg.template(78)
        print(f"| {slack_ver()} > {sbo_url.split('/')[-3].title()} > {self.cyan}{self.name}{self.endc}")
        self.msg.template(78)
        print(f"| {self.green}Package url{self.endc}: {sbo_url}")
        self.msg.template(78)
        print(f"| {self.green}Description: {self.endc}{self.sbo_desc}")
        print(f"| {self.green}SlackBuild: {self.endc}{self.sbo_dwn.split('/')[-1]}")
        print(f"| {self.green}Sources: {self.endc}{', '.join([src.split('/')[-1] for src in self.source_dwn])}")
        print(f"| {self.yellow}Requirements: {self.endc}{', '.join(self.sbo_req)}")
        self.msg.template(78)
        print(f"| {self.red}R{self.endc}{br2}EADME               View the README file")
        print(f"| {self.red}S{self.endc}{br2}lackBuild           View the .SlackBuild file")
        print(f"| In{br1}{self.red}f{self.endc}{br2}o{fix_sp}                View the .info file")
        if "doinst.sh" in self.sbo_files.split():
            print(f"| D{br1}{self.red}o{self.endc}{br2}inst.sh{fix_sp}           View the doinst.sh file")
        print(f"| {self.red}D{self.endc}{br2}ownload             Download this package")
        print(f"| {self.red}B{self.endc}{br2}uild                Download and build")
        print(f"| {self.red}I{self.endc}{br2}nstall              Download/Build/Install")
        print(f"| {self.red}C{self.endc}{br2}lear                Clear screen")
        print(f"| {self.red}Q{self.endc}{br2}uit                 Quit")
        self.msg.template(78)

    def with_checklist(self):
        """Using dialog and checklist option
        """
        data = []
        if not self.name:
            data = self.data
        else:
            for name in self.data:
                if self.name in name:
                    data.append(name)
        if data:
            text = "Press 'spacebar' to choose SlackBuild for view"
            title = " SlackBuilds.org "
            backtitle = f"{_meta_.__all__} {_meta_.__version__}"
            status = False
            pkg = DialogUtil(data, text, title, backtitle, status).checklist()
            if pkg and len(pkg) > 1:
                os.system("clear")
                raise SystemExit("\nslpkg: Error: Choose only one package\n")
            if pkg is None:
                raise SystemExit(1)
            self.name = "".join(pkg)
            os.system("clear")

    def pager(self, text):
        """Read text
        """
        pydoc.pager(text)

    def fill_pager(self, page):
        """Fix pager spaces
        """
        tty_size = os.popen("stty size", "r").read().split()
        rows = int(tty_size[0]) - 1
        lines = sum(1 for line in page.splitlines())
        diff = rows - lines
        fill = "\n" * diff
        if diff > 0:
            return fill
        else:
            return ""

    def error_uns(self):
        """Check if package supported by arch
        before proceed to install
        """
        self.FAULT = ""
        UNST = ["UNSUPPORTED", "UNTESTED"]
        if "".join(self.source_dwn) in UNST:
            self.FAULT = "".join(self.source_dwn)

    def build(self):
        """Only build and create Slackware package
        """
        pkg_security([self.name])
        self.error_uns()
        if self.FAULT:
            print()
            self.msg.template(78)
            print(f"| Package {self.prgnam} {self.red} {self.FAULT} "
                  f"{self.endc}")
            self.msg.template(78)
        else:
            sources = []
            if not os.path.exists(self.meta.build_path):
                os.makedirs(self.meta.build_path)
            if not os.path.exists(self._SOURCES):
                os.makedirs(self._SOURCES)
            os.chdir(self.meta.build_path)
            Download(self.meta.build_path, self.sbo_dwn.split(),
                     repo="sbo").start()
            Download(self._SOURCES, self.source_dwn, repo="sbo").start()
            script = self.sbo_dwn.split("/")[-1]
            for src in self.source_dwn:
                sources.append(src.split("/")[-1])
            BuildPackage(script, sources, self.meta.build_path,
                         auto=False).build()
            slack_package(self.prgnam)  # check if build

    def install(self):
        """Install SBo package found in /tmp directory.
        """
        binary = slack_package(self.prgnam)
        print(f"[ {self.green}Installing{self.endc} ] --> {self.name}")
        PackageManager(binary).upgrade(flag="--install-new")



#!/usr/bin/python3
# -*- coding: utf-8 -*-

# security.py file is part of slpkg.

# Copyright 2014-2022 Dimitris Zlatanidis <d.zlatanidis@gmail.com>
# All rights reserved.

# Slpkg is a user-friendly package manager for Slackware installations

# https://gitlab.com/dslackw/slpkg

# Slpkg is free software: you can redistribute it and/or modify
# it under the terms of the GNU General Public License as published by
# the Free Software Foundation, either version 3 of the License, or
# (at your option) any later version.
# This program is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. See the
# GNU General Public License for more details.
# You should have received a copy of the GNU General Public License
# along with this program. If not, see <http://www.gnu.org/licenses/>.


from slpkg.utils import Utils
from slpkg.messages import Msg


def pkg_security(pkgs):
    """Check packages before install or upgrade for security
    reasons. Configuration file in the /etc/slpkg/pkg_security"""
    packages, msg, utils = [], Msg(), Utils()
    security_packages = utils.read_file("/etc/slpkg/pkg_security")
    for read in security_packages.splitlines():
        read = read.lstrip()
        if not read.startswith("#"):
            packages.append(read.replace("\n", ""))
    for p in pkgs:
        for pkg in packages:
            if p == pkg:
                msg.security_pkg(p)
                if not msg.answer() in ["y", "Y"]:
                    raise SystemExit()
